<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/admin
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_AR_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Enqueue styles for admin areas
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts( $hook_suffix = '' ) {

		global $post_type;

		// Styles for the ads taxonomy pages
		if ( 'edit-tags.php' == $hook_suffix && 'ads' == $post_type ) {

			wp_enqueue_style(
				$this->plugin_name . '-taxonomy-styles',
				plugin_dir_url( __FILE__ ) . 'dist/css/taxonomy-styles.css',
				array(),
				$this->version,
				'all'
			);

		}

		// Styles and scripts for the custom reports screen
		if ( 'ads_page_adsanity-stats' == $hook_suffix ||
			 'ads_page_adsanity-offload-stats' == $hook_suffix ) {

			wp_enqueue_script(
				$this->plugin_name . '-custom_reports',
				plugin_dir_url( __FILE__ ) . 'dist/js/custom-reports.js',
				array( 'jquery', 'adsanity-custom-stats' ),
				$this->version,
				true
			);

		}

	}

	/**
	 * Admin bar styles
	 *
	 * @since    1.0.0
	 */
	public function admin_bar_styles() {

		if ( is_user_logged_in() && current_user_can( 'view_own_adsanity_ads' ) ) {

			wp_enqueue_style(
				$this->plugin_name . '-admin-bar-styles',
				plugin_dir_url( __FILE__ ) . 'dist/css/admin-bar.css',
				array(),
				$this->version,
				'all'
			);

		}

	}

	/**
	 * Create a taxonomy for Advertisers
	 *
	 * @since    1.0.0
	 */
	public function register_taxonomy() {

		$labels = array(
			'name'          => __( 'Advertisers', 'adsanity-ar' ),
			'singular_name' => __( 'Advertiser', 'adsanity-ar' ),
		);

		register_taxonomy(
			'advertiser',
			'ads',
			array(
				'hierarchical'     => true,
				'show_ui'           => true,
				'labels'            => $labels,
				'show_admin_column' => true,
				'description'       => __( 'The ads available to this advertiser', 'adsanity-ar' ),
				'rewrite'           => array(
					'with_front' => false,
					'slug'       => 'advertisers',
				),
			)
		);

	}

	/**
	 * Add Advertiser tp Ad columns
	 *
	 * @since    1.0.1
	 */
	public function columns( $columns ) {

		$columns['adsanity-advertiser'] = __( 'Advertiser', 'adsanity-ar' );
		return $columns;

	}

	/**
	 * Add Advertiser column to sortable columns
	 *
	 * @since    1.0.1
	 */
	public function sortable_columns( $columns ) {

		$columns['adsanity-advertiser'] = 'adsanity-advertiser';
		return $columns;

	}

	/**
	 * Get column values for Advertiser column
	 *
	 * @since    1.0.1
	 */
	public function column_values( $value, $column, $post_id ) {

		$advertisers = get_the_terms( $post_id, 'advertiser' );

		if ( ! $advertisers ) {
			return;
		}

		$return = '';

		foreach ( $advertisers as $key => $advertiser ) {
			$return .= esc_html( $advertiser->name );

			if ( count( $advertisers ) > $key + 1 ) {
				$return .= ', ';
			}
		}

		return $return;

	}

	/**
	 * Sort the column by advertiser
	 *
	 * @since    1.0.1
	 */
	public function sort_by_advertiser( $clauses, $wp_query ) {

		global $wpdb;
		if ( is_admin() && isset( $wp_query->query['orderby'] ) && $wp_query->query['orderby'] == 'adsanity-advertiser' ) {

			$clauses['join'] .= "
			LEFT OUTER JOIN {$wpdb->term_relationships} ON {$wpdb->posts}.ID={$wpdb->term_relationships}.object_id
			LEFT OUTER JOIN {$wpdb->term_taxonomy} USING (term_taxonomy_id)
			LEFT OUTER JOIN {$wpdb->terms} USING (term_id)
			";

			$clauses['where'] .= "AND (taxonomy = 'advertiser')";
			$clauses['groupby'] = "object_id";
			$clauses['orderby'] = "GROUP_CONCAT({$wpdb->terms}.name ORDER BY name ASC)";

			if ( strtoupper( $wp_query->get( 'order' ) ) == 'ASC' ) {
				$clauses['orderby'] .= 'ASC';
			} else {
				$clauses['orderby'] .= 'DESC';
			}
		}

		return $clauses;

	}

	/**
	 * Create an admin bar link
	 *
	 * @since    1.0.0
	 */
	public function admin_bar( $admin_bar ) {

		if ( ! current_user_can( 'view_own_adsanity_ads' ) ) {
			return;
		}

		if ( function_exists( 'run_adsanity_gati' ) ) {
			$page = 'adsanity-offload-stats';
		} else {
			$page = 'adsanity-stats';
		}

		// Get the 'My Ads' page url
		$my_ads_url = add_query_arg( array(
			'page' => $page,
			), admin_url( 'admin.php' )
		);

		$my_ads_url = esc_url( $my_ads_url );

		// Add a menu item
		$admin_bar->add_menu( array(
			'id'    => 'adsanity-ar-my-ads',
			'title' => __( 'Advertiser Dashboard', 'adsanity-ar' ),
			'href'  => $my_ads_url
		) );

		// Remove the dashboard menu link
		if ( ! is_admin() ) {
			$admin_bar->remove_menu( 'site-name' );
		}

		$admin_bar->remove_menu( 'wp-logo' );

	}

	/**
	 * Turn transients off for stats dashboard
	 *
	 * @since    1.0.0
	 * @param    bool    $override    Are transients overridden?
	 */
	public function remove_stats_dashboard_transients( $override ) {

		if ( ! current_user_can( 'advertiser' ) ) {
			return $override;
		}

		return true;

	}

	/**
	 * Modify the arguments for retrieving ads
	 *
	 * @since    1.0.0
	 * @param    array    $args    The current arguments for stats dashboard
	 */
	public function modify_ad_arguments( $args ) {

		if ( ! is_admin() || ! current_user_can( 'advertiser' ) ) {
			return $args;
		}

		// Get current user id
		$user_id = get_current_user_id();

		// Get the term related to this user id
		$term_args = array(
			'taxonomy'   => 'advertiser',
			'hide_empty' => false,
			'fields'     => 'ids',
			'meta_query' => array(
				array(
					'key'     => 'user_id',
					'value'   => $user_id,
					'compare' => '=',
				),
			),
		);

		$terms = new WP_Term_Query( $term_args );

		if ( empty( $terms->terms ) || count( $terms->terms ) > 1 ) {
			$term_id = false;
		}

		$term_id = $terms->terms[0];

		// Add tax query to the ad WP_Query
		$args['tax_query'] = array(
			array(
				'taxonomy' => 'advertiser',
				'field'    => 'term_id',
				'terms'    => $term_id,
			),
		);

		return $args;

	}

	/**
	 * Add an advertiser filter to custom stats
	 *
	 * @since    1.0.0
	 */
	public function custom_stats_advertiser() {

		require_once( plugin_dir_path( __FILE__ ) . 'partials/custom-reports-toggle.php' );
	}

	/**
	 * Change the GATI reports page configuration message
	 *
	 * @since    1.0.0
	 * @param    string    $message    The configuration message
	 */
	public function gati_configuration_message( $message ) {

		if ( current_user_can( 'advertiser' ) ) {
			$message = sprintf( '<p>%s</p>',
				esc_html__( 'Stats have not been configured. Please contact your site administrator', 'adsanity-ar' )
			);
		}

		return $message;

	}

	/**
	 * Change the GATI custom reports to always construct a filter
	 *
	 * @since
	 * @param    bool    $bool    Construct filter or not
	 */
	public function construct_filter( $bool ) {

		if ( ! current_user_can( 'advertiser' ) ) {
			return $bool;
		}

		return true;

	}

	/**
	 * Add filters for advertiser GATI dashboard
	 *
	 * @since    1.0.0
	 * @param    string    $filter    The filter for Google Analytics call
	 */
	public function gati_dashboard_filter( $filter ) {

		if ( ! current_user_can( 'advertiser' ) ) {
			return $filter;
		}


		// Get this user's ads

		// Get current user id
		$user_id = get_current_user_id();

		// Get the term related to this user id
		$term_args = array(
			'taxonomy'   => 'advertiser',
			'hide_empty' => false,
			'fields'     => 'ids',
			'meta_query' => array(
				array(
					'key'     => 'user_id',
					'value'   => $user_id,
					'compare' => '=',
				),
			),
		);

		$terms = new WP_Term_Query( $term_args );

		if ( empty( $terms->terms ) || count( $terms->terms ) > 1 ) {
			return $filter;
		}

		$term_id = $terms->terms[0];

		// Get the ads for this advertiser
		$ad_args = array(
			'post_type'      => 'ads',
			'posts_per_page' => -1,
			'fields'         => 'ids',
			'tax_query'      => array(
				array(
					'taxonomy' => 'advertiser',
					'field'    => 'term_id',
					'terms'    => $term_id,
				),
			),
		);

		$ads = new WP_Query( $ad_args );

		$ads_key = 0;

		if ( $ads->have_posts() ) {
			while ( $ads->have_posts() ) {
				$ads->the_post();

				// Add to the filters param
				if ( $ads_key === 0 ) {
					$filter .= ';ga:eventLabel==' . get_the_title();
				} else {
					$filter .= ',ga:eventLabel==' . get_the_title();
				}

				++$ads_key;
			}
		}

		wp_reset_postdata();

		return $filter;

	}

	/**
	 * Do not show ad group stats block for advertisers
	 *
	 * @since    1.0.0
	 * @param    bool    $bool    Whether or not to show the stats block
	 */
	public function ad_group_dashboard( $bool ) {

		if ( ! current_user_can( 'advertiser' ) ) {
			return $bool;
		}

		return false;

	}

	/**
	 * Adds the changelog to AdSanity's main changelog page.
	 *
	 * @since 1.3
	 */
	public static function changelog() {
		require_once plugin_dir_path( __FILE__ ) . 'partials/changelog.php';
	}

}
