<?php

/**
 * The email-specific functionality of the plugin.
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/email
 */

/**
 * The email-specific functionality of the plugin.
 *
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/email
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_AR_Emails {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The plugin options
	 *
	 * @since    1.0.0
	 * @access    private
	 * @var    array    $options    The options of this plugin
	 */
	private $options;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->set_options();

	}

	/**
	 * Retrieve the plugin options
	 *
	 * @since    1.0.0
	 */
	private function set_options() {

		$this->options = get_option( 'adsanity-advertiser-reporting-options' );

	}

	/**
	 * Parse the email to insert variables
	 *
	 * @since    1.0.0
	 * @param    string    $email_template    the email message template
	 * @param    string    $user_id    the user name
	 * @param    string    $ad_name    the ad's name
	 * @param    int    $days    The number of days until expiration
	 * @param    string    $date    The date of ad expiration
	 */
	private function parse_email( $email_template, $user_name = false, $ad_name, $days, $date ) {

		if ( empty( $email_template ) ) {
			return;
		}

		if ( ! $user_name ) {
			$user_name = $this->options['default-name'];
		}

		$email = str_replace( '*|NAME|*', $user_name, $email_template );
		$email = str_replace( '*|AD|*', $ad_name, $email );
		$email = str_replace( '*|DAYS|*', $days, $email );
		$email = str_replace( '*|DATE|*', $date, $email );

		return $email;

	}

	/**
	 * Endpoint to send a test email
	 */
	public function test_email() {

		$user_id        = sanitize_text_field( $_POST['user_id'] );
		$ad_id          = intval( $_POST['ad_id'] );
		$user_email     = sanitize_text_field( $_POST['user_email'] );
		$email_template = wp_kses_post( stripslashes( $_POST['email_template'] ) );
		$days           = intval( $_POST['days'] );
		$email_subject  = sanitize_text_field( $_POST['email_subject'] );
		$default_name   = sanitize_text_field( $_POST['default_name'] );


		$email = $this->send_email( array(
			'user_id'        => $user_id,
			'ad_id'          => $ad_id,
			'user_email'     => $user_email,
			'days'           => $days,
			'email_subject'  => $email_subject,
			'email_template' => $email_template,
			'default_name'   => $default_name,
		) );

		if ( is_wp_error( $email ) ) {
			wp_send_json_error( $email->get_error_message(), $email->get_error_code() );
			return;
		}

		wp_send_json_success();
		return;

	}

	/**
	 * Send an email reminder
	 *
	 * @since    1.0.0
	 * @param    array    $args    Arguments for email parsing
	 * @return    mixed    Either WP_Error object or null if successful
	 */
	public function send_email( $args = array() ) {

		/**
		 * Required:
		 *     user_id
		 *     ad_id
		 */
		$defaults = array(
			'user_email'     => false,
			'email_subject'  => false,
			'email_template' => false,
			'days'           => false,
			'default_name'   => false,
		);

		// Extract values from $args, taking into account defaults
		$values = wp_parse_args( $args, $defaults );

		$user_email     = $values['user_email'];
		$email_subject  = $values['email_subject'];
		$email_template = $values['email_template'];
		$days           = $values['days'];
		$default_name   = $values['default_name'];

		if ( ! isset( $values['user_id'] ) ) {
			return new WP_Error( 400, __( 'User id not passed.', 'adsanity-ar' ) );
		}

		if ( ! isset( $values['ad_id'] ) ) {
			return new WP_Error( 400, __( 'Ad id not passed.', 'adsanity-ar' ) );
		}

		$user_id = intval( $values['user_id'] );
		$ad_id = intval( $values['ad_id'] );

		// Retrieve user data
		$user = get_userdata( $user_id );

		// Initially set to default name
		$user_name = $default_name;

		if ( ! $user instanceof WP_User || ! $user->exists() ) {
			return new WP_Error( 400, __( 'User not found or invalid.', 'adsanity-ar' ) );
		}

		if ( $user->get( 'display_name' ) ) {
			$user_name = $user->get( 'display_name' );
		}

		if ( ! $user_email ) {
			$user_email = $user->get( 'user_email' );
		}

		$ad_name = get_the_title( $ad_id );
		$date    = get_post_meta( $ad_id, '_end_date', true );
		$date    = date( 'd F Y', $date );

		if ( ! $days ) {
			$now        = new DateTime();
			$expiration = new DateTime( $date );
			$interval   = $now->diff( $expiration );
			$days       = $interval->format( '%d' );
		}

		// If email subject is not passed into this method, use option
		if ( ! $email_subject ) {
			$email_subject = $this->options['email-subject'];
		}

		// Use helper method to parse the email message
		if ( ! $email_template ) {
			$email_template = $this->options['email-template'];
		}
		$email_message = $this->parse_email( $email_template, $user_name, $ad_name, $days, $date );

		$email_message = apply_filters( 'adsanity_advertiser_reporting_email_message', $email_message, $ad_id );

		// Send mail
		$mail = wp_mail(
			$user_email,
			$email_subject,
			$email_message,
			array( 'Content-Type: text/html; charset=UTF-8' )
		);

		if ( ! $mail ) {
			return new WP_Error( 500, __( 'Failed to send email.', 'adsanity-ar' ) );
		}

		return;

	}

	/**
	 * When an ad is saved, check to see if we need to update the email cron
	 *
	 * @since    1.0.0
	 * @param    int    $post_id    The post's id
	 */
	public function update_email_cron( $post_id ) {

		// Return if not admin or not ad post type
		if ( ! is_admin() || ! isset( $_POST['post_type'] ) ||
			$_POST['post_type'] != 'ads' || ! isset( $_POST['end_date'] ) ) {
			return;
		}

		$end_date = intval( $_POST['end_date'] );

		// If this ad is forever, return
		if ( $end_date == ADSANITY_EOL ) {
			return;
		}

		// Get the advertiser, if any, associated with this ad
		$advertisers = get_the_terms( $post_id, 'advertiser' );

		if ( empty( $advertisers ) ) {
			return;
		}

		foreach ( $advertisers as $advertiser ) {

			// Get the term meta user id
			$user_id = get_term_meta( $advertiser->term_id, 'user_id', true );

			// The arguments for cron
			$cron_args = array(
				'user_id' => $user_id,
				'ad_id'   => $post_id,
			);

			// Delete event
			wp_clear_scheduled_hook( 'adsanity_ar_send_email', $cron_args );

			// Get the end date
			$days = $this->options['timing'];

			// Schedule event
			wp_schedule_single_event( $end_date - ( $days * DAY_IN_SECONDS ), 'adsanity_ar_send_email', $cron_args );

		}

	}

	/**
	 * After the plugin options have been updated, reschedule the email
	 *
	 * @param    array    $old_value    The old value
	 * @param    array    $new_value    The new value
	 * @since    1.0.0
	 */
	public function options_updated( $old_value, $new_value ) {

		// If the timing has not been updated, return
		if ( $old_value['timing'] === $new_value['timing'] ) {
			return;
		}

		// Get all the advertiser terms
		$advertisers = new WP_Term_Query( array(
			'taxonomy'   => 'advertiser',
			'hide_empty' => true,
			'fields'     => 'ids',
		) );

		if ( ! isset( $advertisers->terms ) ) {
			return;
		}

		// Array of term ids
		$advertisers = $advertisers->terms;

		// Loop over each advertiser
		foreach( $advertisers as $advertiser ) {

			$user_id = get_term_meta( $advertiser, 'user_id', true );

			// Get all ads with this term
			$ad_args = array(
				'post_type'      => 'ads',
				'posts_per_page' => -1,
				'fields'         => 'ids',
				'tax_query'      => array(
					array(
						'taxonomy' => 'advertiser',
						'field'    => 'term_id',
						'terms'    => $advertiser,
					),
				),
			);

			$ads = new WP_Query( $ad_args );

			// Loop over each ad
			if ( $ads->have_posts() ) {
				while ( $ads->have_posts() ) {
					$ads->the_post();

					$ad_id = get_the_ID();

					// The arguments for cron
					$cron_args = array(
						'user_id' => $user_id,
						'ad_id'   => $ad_id,
					);

					// Delete event
					wp_clear_scheduled_hook( 'adsanity_ar_send_email', $cron_args );

					$end_date = get_post_meta( $ad_id, '_end_date', true );

					// If this ad is forever, return
					if ( $end_date == ADSANITY_EOL ) {
						return;
					}

					// Get the end date
					$days = $this->options['timing'];

					// Schedule event
					wp_schedule_single_event( $end_date - ( $days * DAY_IN_SECONDS ), 'adsanity_ar_send_email', $cron_args );

				}
			}

			wp_reset_postdata();

		}

	}

}
