<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/includes
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_AR {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Adsanity_AR_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		$this->plugin_name = 'adsanity-ar';
		$this->version = '1.4';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_email_hooks();
		$this->define_settings_hooks();
		$this->define_user_role_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Adsanity_AR_Loader. Orchestrates the hooks of the plugin.
	 * - Adsanity_AR_i18n. Defines internationalization functionality.
	 * - Adsanity_AR_Admin. Defines all hooks for the admin area.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-adsanity-ar-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-adsanity-ar-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-adsanity-ar-admin.php';

		/**
		 * The class responsible for defining all actions that relate to emails.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'emails/class-adsanity-ar-emails.php';

		/**
		 * The class responsible for defining all actions that relate to settings.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'settings/class-adsanity-ar-settings.php';

		/**
		 * The class responsible for user role functionality
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'user-role/class-adsanity-ar-user-role.php';

		/**
		 * Update engine
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'update-engine/remote-api.php';

		$this->loader = new Adsanity_AR_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Adsanity_AR_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Adsanity_AR_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Adsanity_AR_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts', 10, 1 );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'admin_bar_styles' );
		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_admin, 'admin_bar_styles' );
		$this->loader->add_action( 'init', $plugin_admin, 'register_taxonomy' );
		$this->loader->add_action( 'adsanity_ads_sortable_posts_columns', $plugin_admin, 'sortable_columns', 10, 1 );
		$this->loader->add_filter( 'adsanity_ads_posts_columns', $plugin_admin, 'columns', 10, 1 );
		$this->loader->add_filter( 'adsanity_ads_posts_columns_adsanity-advertiser_value', $plugin_admin, 'column_values', 10, 3 );
		$this->loader->add_filter( 'posts_clauses', $plugin_admin, 'sort_by_advertiser', 10, 2 );
		$this->loader->add_action( 'admin_bar_menu', $plugin_admin, 'admin_bar', 100, 1 );
		$this->loader->add_filter( 'adsanity_override_total_stats_transient', $plugin_admin, 'remove_stats_dashboard_transients', 10, 1 );
		$this->loader->add_filter( 'adsanity_total_stats_args', $plugin_admin, 'modify_ad_arguments', 10, 1 );
		$this->loader->add_filter( 'adsanity_get_all_ads_args', $plugin_admin, 'modify_ad_arguments', 10, 1 );
		$this->loader->add_filter( 'adsanity_get_ads_args', $plugin_admin, 'modify_ad_arguments', 10, 1 );
		$this->loader->add_action( 'adsanity_custom_stats_before_ad_selection', $plugin_admin, 'custom_stats_advertiser' );
		$this->loader->add_filter( 'adsanity_gati_configuration_message', $plugin_admin, 'gati_configuration_message', 10, 1 );
		$this->loader->add_filter( 'adsanity_gati_construct_custom_reports_filter', $plugin_admin, 'construct_filter', 10, 1 );
		$this->loader->add_filter( 'adsanity_gati_reports_dashboard_filters', $plugin_admin, 'gati_dashboard_filter', 10, 1 );
		$this->loader->add_filter( 'adsanity_show_ad_group_block', $plugin_admin, 'ad_group_dashboard', 10, 1 );
		$this->loader->add_action( 'adsanity_changelog_screen', $plugin_admin, 'changelog' );

	}

	/**
	 * Register all of the hooks related to the email area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_email_hooks() {

		$plugin_email = new Adsanity_AR_Emails( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'adsanity_ar_send_email', $plugin_email, 'send_email', 10, 2 );
		$this->loader->add_action( 'save_post', $plugin_email, 'update_email_cron' );
		$this->loader->add_action( 'update_option_adsanity-advertiser-reporting-options', $plugin_email, 'options_updated', 10, 2 );
		$this->loader->add_action( 'wp_ajax_adsanity_ar_test_email', $plugin_email, 'test_email' );

	}

	/**
	 * Register all of the hooks related to the email area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_settings_hooks() {

		$plugin_settings = new Adsanity_AR_Settings( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'adsanity-addons', $plugin_settings, 'add_settings_tab' );
		$this->loader->add_action( 'admin_init', $plugin_settings, 'register_settings' );
		$this->loader->add_action( 'admin_init', $plugin_settings, 'register_sections' );
		$this->loader->add_action( 'admin_init', $plugin_settings, 'register_fields' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_settings, 'scripts', 10, 1 );

	}

	/**
	 * Register all of the hooks related to the user role functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_user_role_hooks() {

		$plugin_user = new Adsanity_AR_User_Role( $this->get_plugin_name(), $this->get_version() );
		$this->loader->add_action( 'user_register', $plugin_user, 'create_edit_term', 10, 1 );
		$this->loader->add_action( 'delete_user', $plugin_user, 'delete_term', 10, 1 );
		$this->loader->add_action( 'profile_update', $plugin_user, 'update_user', 10, 2 );
		$this->loader->add_action( 'delete_user_form', $plugin_user, 'user_deletion_warning', 10, 2 );
		$this->loader->add_filter( 'login_redirect', $plugin_user, 'login_redirect', 10, 3 );
		$this->loader->add_action( 'admin_init', $plugin_user, 'add_capability' );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Adsanity_AR_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
