<?php

/**
 * The settings-specific functionality of the plugin.
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/settings
 */

/**
 * The settings-specific functionality of the plugin.
 *
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/settings
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_AR_Settings {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The plugin options
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      array    $options    The options of this plugin.
	 */
	private $options;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->set_options();

	}

	/**
	 * Add options to AdSanity Add-Ons Settings tab
	 *
	 * @since    1.0.0
	 */
	public function add_settings_tab( $addons = array() ) {

		$addons[] = array(
			'slug' => 'advertiser-reporting',
			'name' => __( 'Advertiser Reporting', 'adsanity-ar' ),
		);

		return $addons;

	}

	/**
	 * Sets the class variable $options
	 */
	private function set_options() {

		$this->options = get_option( 'adsanity-advertiser-reporting-options', array() );

	}

	/**
	 * Add error message
	 */
	private function add_error( $key, $message ) {

		add_settings_error(
			'adsanity-advertiser-reporting-options',
			$key,
			$message,
			'error'
		);

	}

	/**
	 * Register plugin settings
	 */
	public function register_settings() {

		register_setting(
			'adsanity-advertiser-reporting-options',
			'adsanity-advertiser-reporting-options',
			array( $this, 'validate_options' )
		);

	}

	/**
	 * Register settings sections
	 */
	public function register_sections() {

		add_settings_section(
			$this->plugin_name . '-options',
			sprintf( __( '%s Advertiser Reporting', 'adsanity-raw' ), 'AdSanity' ),
			array( $this, 'section_view' ),
			'adsanity-advertiser-reporting-options'
		);

	}

	/**
	 * Creates a settings section
	 */
	public function section_view( $params ) {

		include( plugin_dir_path( __FILE__ ) . 'partials/adsanity-ar-admin-section-view.php' );

	}

	/**
	 * Register settings fields
	 */
	public function register_fields() {

		add_settings_field(
			'default-name',
			esc_html__( 'Default Name', 'adsanity-ar' ),
			array( $this, 'default_name_cb' ),
			'adsanity-advertiser-reporting-options',
			$this->plugin_name . '-options'
		);

		add_settings_field(
			'email-subject',
			esc_html__( 'Email Subject', 'adsanity-ar' ),
			array( $this, 'email_subject_cb' ),
			'adsanity-advertiser-reporting-options',
			$this->plugin_name . '-options'
		);

		add_settings_field(
			'email-template',
			esc_html__( 'Email Template', 'adsanity-ar' ),
			array( $this, 'email_template_cb' ),
			'adsanity-advertiser-reporting-options',
			$this->plugin_name . '-options'
		);

		add_settings_field(
			'when-to-send',
			esc_html__( 'Timing', 'adsanity-ar' ),
			array( $this, 'timing_cb' ),
			'adsanity-advertiser-reporting-options',
			$this->plugin_name . '-options'
		);

	}

	/**
	 * Callback for default name field
	 *
	 * @since    1.0.0
	 */
	public function default_name_cb() {

		$default = '';

		if ( isset( $this->options['default-name'] ) ) {
			$default = $this->options['default-name'];
		}

		printf( '<input type="text" name="adsanity-advertiser-reporting-options[default-name]" value="%s" />',
			esc_attr( $default )
		);

	}

	/**
	 * Callback for email subject field
	 *
	 * @since    1.0.0
	 */
	public function email_subject_cb() {

		$default = '';

		if ( isset( $this->options['email-subject'] ) ) {
			$default = $this->options['email-subject'];
		}

		printf( '<input class="widefat" value="%s" type="text" name="adsanity-advertiser-reporting-options[email-subject]" />',
			esc_attr( $default )
		);

	}

	/**
	 * Callback for email template field
	 *
	 * @since    1.0.0
	 */
	public function email_template_cb() {

		$default = '';

		if ( isset( $this->options['email-template'] ) ) {
			$default = $this->options['email-template'];
		}

		$settings = array(
			'textarea_name' => esc_attr( 'adsanity-advertiser-reporting-options[email-template]' ),
		);

		wp_editor( $default, 'adsanity_ar_email_template', $settings );

	}

	/**
	 * Callback for timing field
	 *
	 * @since    1.0.0
	 */
	public function timing_cb() {

		$input_id = 'adsanity-ar-timing';

		// sprintf();

		$input = sprintf( '<input value="%s" type="number" id="%s" name="%s" />',
			intval( $this->options['timing'] ),
			$input_id,
			'adsanity-advertiser-reporting-options[timing]'
		);

		printf(
			__( 'Send an email %s days before ad expiration', 'adsanity-ar' ),
			$input
		);

	}

	/**
	 * Validate options
	 *
	 * @since    1.0.0
	 */
	public function validate_options( $dirty = array() ) {

		$options = wp_parse_args(
			$this->options,
			array(
				'email-template' => '',
			)
		);

		if ( ! isset( $_POST['adsanity-advertiser-reporting-options']['default-name'] ) ) {
			unset( $options['default-name'] );
		} else {
			$options['default-name'] = sanitize_text_field( $_POST['adsanity-advertiser-reporting-options']['default-name'] );
		}

		if ( ! isset( $_POST['adsanity-advertiser-reporting-options']['email-subject'] ) ) {
			unset( $options['email-subject'] );
		} else {
			$options['email-subject'] = sanitize_text_field( $_POST['adsanity-advertiser-reporting-options']['email-subject'] );
		}

		if ( ! isset( $_POST['adsanity-advertiser-reporting-options']['email-template'] ) ) {
			unset( $options['email-template'] );
		} else {
			$options['email-template'] = wp_kses_post( stripslashes( $_POST['adsanity-advertiser-reporting-options']['email-template'] ) );
		}

		if ( ! isset( $_POST['adsanity-advertiser-reporting-options']['timing'] ) ) {
			unset( $options['timing'] );
		} else {
			$options['timing'] = intval( $_POST['adsanity-advertiser-reporting-options']['timing'] );
		}

		$clean = apply_filters( 'adsanity_advertiser_reporting_validate_options', $options, $dirty );

		return $clean;

	}

	/**
	 * Enqueue settings scripts
	 *
	 * @since    1.1.0
	 */
	public function scripts( $hook_suffix ) {

		if ( 'ads_page_adsanity-settings' !== $hook_suffix ) {
			return;
		}

		wp_enqueue_script(
			$this->plugin_name . '-settings',
			plugin_dir_url( __FILE__ ) . 'dist/js/settings.js',
			array( 'jquery', 'jquery-ui-core', 'jquery-ui-autocomplete', ),
			$this->version,
			true
		);

		wp_localize_script(
			$this->plugin_name . '-settings',
			'ADSANITY_AR_REST_VARS',
			array(
				'endpoint' => esc_js( trailingslashit( get_rest_url() ) . 'wp/v2' ),
			)
		);

		wp_enqueue_style(
			$this->plugin_name . '-settings',
			plugin_dir_url( __FILE__ ) . 'dist/css/settings.css',
			array(),
			$this->version,
			'all'
		);

	}

}
