<?php

/**
 * The user role functionality of the plugin.
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/user_role
 */

/**
 * The user role functionality of the plugin.
 *
 * @package    Adsanity_AR
 * @subpackage Adsanity_AR/user_role
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_AR_User_Role {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Create a new term every time a new advertiser user is created
	 *
	 * @param    int    $user_id    The user that was registered
	 * @since    1.0.0
	 */
	public function create_edit_term( $user_id ) {

		$user = get_userdata( $user_id );

		// Check to see if this user is an advertiser
		$roles = $user->roles;
		if ( ! in_array( 'advertiser', $roles ) ) {
			return;
		}

		$term = $user->display_name;

		$description = '';
		if ( ! empty( $user->user_email ) &&
			 ! empty( $user->user_url ) ) {
			$description .= sprintf(
				'%s: %s
%s: %s',
				esc_html__( 'Email', 'adsanity-ar' ),
				$user->user_email,
				esc_html__( 'Website', 'adsanity-ar' ),
				$user->user_url
			);
		} elseif ( ! empty( $user->user_email ) ) {
			$description .= sprintf(
				'%s: %s',
				esc_html__( 'Email', 'adsanity-ar' ),
				$user->user_email
			);
		} elseif ( ! empty( $user->user_url ) ) {
			$description .= sprintf(
				'%s: %s',
				esc_html__( 'Website', 'adsanity-ar' ),
				$user->user_url
			);
		}

		// Check for a pre-existing term
		if ( $existing_term = get_term_by( 'slug', $term, 'advertiser' ) ) {

			// Updates term description
			wp_update_term( $existing_term->term_id, 'advertiser', [
				'description' => $description,
			] );

			// Updates user id
			update_term_meta( $existing_term->term_id, 'user_id', $user_id, true );
			return;
		}

		// Create term with description
		$create_term = wp_insert_term( $term, 'advertiser', array(
			'description' => $description,
		) );

		// Add user id to term meta
		add_term_meta( $create_term['term_id'], 'user_id', $user_id, true );

	}

	/**
	 * Delete a new term every time a new advertiser user is deleted
	 *
	 * @param    int    $user_id    The user that was registered
	 * @since    1.0.0
	 */
	public function delete_term( $user_id ) {

		// Get the term by the advertiser user id
		$args = array(
			'taxonomy'   => 'advertiser',
			'hide_empty' => false,
			'fields'     => 'ids',
			'meta_query' => array(
				array(
					'key'     => 'user_id',
					'value'   => $user_id,
					'compare' => '=',
				),
			),
		);

		$terms = new WP_Term_Query( $args );

		if ( empty( $terms->terms ) ) {
			return;
		}

		wp_delete_term( $terms->terms[0], 'advertiser' );

	}

	/**
	 * Create/Delete term when a user's role has been updated
	 *
	 * @since    1.0.0
	 * @param    integer    $user_id    The user ID of the user being edited.
	 * @param    object    $old_user_data    The WP_User object of the user before it was edited
	 */
	public function update_user( $user_id, $old_user_data ) {

		// The array of old user roles
		$old_user_roles = $old_user_data->allcaps;

		// The new user
		$new_user = get_userdata( $user_id );
		$new_user_data = $new_user->data;

		// The array of new user roles
		$new_user_roles = $new_user->allcaps;

		// If both old and new do not have advertisers, do not update
		if ( ! array_key_exists( 'advertiser', $old_user_roles ) &&
			 ! array_key_exists( 'advertiser', $new_user_roles ) ) {
			return;
		}

		// If both old and new have advertisers, do not update
		if ( array_key_exists( 'advertiser', $old_user_roles ) &&
			 array_key_exists( 'advertiser', $new_user_roles ) ) {

			// If the old and new user have emails changed, update user
			if ( $old_user_data->user_email !== $new_user_data->user_email ) {
				$this->create_edit_term( $user_id );
				return;
			}

			return;
		}

		// Create a term if role has changed to advertiser
		if ( array_key_exists( 'advertiser', $new_user_roles ) ) {
			$this->create_edit_term( $user_id );
			return;
		}

		// Delete a term if role has changed from advertiser
		if ( ! array_key_exists( 'advertiser', $new_user_roles ) ) {
			$this->delete_term( $user_id );
			return;
		}

	}

	/**
	 * Add a warning to user deletion
	 *
	 * @param    WP_User    $current_user    WP_User object for the current user.
	 * @param    array    $users    Array of IDs for users being deleted.
	 * @since    1.0.0
	 */
	public function user_deletion_warning( $current_user, $user_ids ) {

		$advertiser_present = false;

		foreach ( $user_ids as $id ) {
			$user_roles = get_userdata( $id )->roles;

			if ( in_array( 'advertiser', $user_roles ) ) {
				$advertiser_present = true;
				break;
			}
		}

		if ( ! $advertiser_present ) {
			return;
		}

		printf( '<p><strong>%s</strong> %s</p>',
			esc_html__( 'Warning!', 'adsanity-ar' ),
			esc_html__( 'Deleting this user will also delete the advertiser taxonomy term associated with this advertiser.', 'adsanity-ar' )
		);

	}

	/**
	 * Redirect advertisers to their dashboard on login
	 *
	 * @since    1.0.0
	 */
	public function login_redirect( $redirect_to, $request, $user ) {

		// Is this the GATI page or normal page?
		if ( function_exists( 'run_adsanity_gati' ) ) {
			$page = 'adsanity-offload-stats';
		} else {
			$page = 'adsanity-stats';
		}

		if ( isset( $user->roles ) && is_array( $user->roles ) ) {
			if ( in_array( 'advertiser', $user->roles ) ) {
				$url = add_query_arg( array(
					'page' => $page,
					), admin_url( 'admin.php' )
				);

				$url = esc_url( $url );
				return $url;
			}
		}

		return $redirect_to;

	}

	/**
	 * Add adsanity_view_ad_stats capability to advertiser
	 *
	 * @since    1.0.0
	 */
	public function add_capability() {

		$advertiser = get_role( 'advertiser' );
		$advertiser->add_cap( 'adsanity_view_ad_stats' );
		$advertiser->add_cap( 'read' );

	}

}
