<?php

/**
 * The admin ajax functionality of the plugin.
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/admin
 */

/**
 * The admin ajax functionality of the plugin.
 *
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/admin
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_Caa_Admin_Ajax {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Autocomplete post by name
	 * This is used for posts, pages, and CPT posts
	 *
	 * @since    1.0.0
	 */
	public function autocomplete_post() {

		if ( ! is_admin() ) {
			return;
		}

		$input = sanitize_text_field( $_GET['autocomplete'] );
		$post_type = sanitize_text_field( $_GET['post_type'] );

		if ( $post_type === 'page_parent' ) {
			$post_type = 'page';
		}

		$return = array();

		$posts = get_posts( array(
			'post_type'      => $post_type,
			's'              => $input,
			'posts_per_page' => 3
		) );

		foreach ( $posts as $post ) {
			array_push( $return, array(
				'value' => $post->post_title,
				'id'    => $post->ID,
			) );
		}

		wp_send_json_success( $return );

	}

	/**
	 * Autocomplete user by name
	 *
	 * @since    1.0.0
	 */
	public function autocomplete_user() {

		if ( ! is_admin() ) {
			return;
		}

		$input = sanitize_text_field( $_GET['autocomplete'] );

		$return = array();

		$users = new WP_User_Query( array(
			'search'         => '*' . esc_attr( $input ) . '*',
			'search_columns' => array(
				'user_login',
				'user_nicename',
				'user_email',
				'user_url',
			),
		) );

		$users = $users->get_results();

		foreach ( $users as $user ) {
			array_push( $return, array(
				'value' => $user->data->user_nicename,
				'id'    => $user->data->ID,
			) );
		}

		wp_send_json_success( $return );

	}

	/**
	 * Autocomplete term by name
	 *
	 * @since    1.0.0
	 */
	public function autocomplete_term() {

		if ( ! is_admin() ) {
			return;
		}

		$input = sanitize_text_field( $_GET['autocomplete'] );

		$return = array();

		$terms = get_terms( array(
			'hide_empty' => false,
			'name__like' => $input,
		) );

		foreach ( $terms as $term ) {

			// Create a string label
			$label = esc_html( $term->name );

			global $wp_taxonomies;

			// Add the taxonomy and post types to results
			if ( isset( $wp_taxonomies[ $term->taxonomy ] ) ) {

				$label .= sprintf( ' - %s ',
					esc_html( $wp_taxonomies[ $term->taxonomy ]->label )
				);

				$post_type_names = array();

				foreach ( $wp_taxonomies[ $term->taxonomy ]->object_type as $post_type ) {
					array_push( $post_type_names, get_post_type_object( $post_type )->labels->name );
				}

				$label .= sprintf( '(%s) ',
					implode( ', ', $post_type_names )
				);
			}

			$id = sprintf( '%s,%s',
				esc_attr( $term->term_id ),
				esc_attr( $term->taxonomy )
			);

			array_push( $return, array(
				'value' => $label,
				'id'    => $id,
			) );

		}

		wp_send_json_success( $return );

	}

	/**
	 * Get all available post types
	 *
	 * @since    1.0.0
	 */
	public function get_post_type() {

		$post_types = get_post_types( array(
			'public' => true,
		) );

		// Exclude ads
		unset( $post_types[ 'ads' ] );

		$return = array();

		foreach ( $post_types as $key => $val ) {
			$name = get_post_type_object( $key )->labels->name;
			$return[ $key ] = $name;
		}

		wp_send_json_success( $return );

	}

	/**
	 * Get all available user types
	 *
	 * @since    1.0.0
	 */
	public function get_user_type() {

		$user_types = get_editable_roles();

		$return = array();

		foreach( $user_types as $key => $value ) {
			$return[ $key ] = $value['name'];
		}

		wp_send_json_success( $return );

	}

	/**
	 * Get all available post formats
	 *
	 * @since    1.0.0
	 */
	public function get_post_format() {

		$post_formats = get_post_format_strings();

		wp_send_json_success( $post_formats );

	}

	/**
	 * Get all available page types
	 *
	 * @since    1.0.0
	 */
	public function get_page_type() {

		$return = array(
			'front_page' => esc_html__( 'Front Page', 'adsanity-caa' ),
			'posts_page' => esc_html__( 'Posts Page', 'adsanity-caa' ),
			'top_level'  => esc_html__( 'Top Level Page (has no parent)', 'adsanity-caa' ),
			'parent'     => esc_html__( 'Parent Page (has children)' ),
			'child'      => esc_html__( 'Child Page (has a parent)', 'adsanity-caa' ),
		);

		wp_send_json_success( $return );

	}

	/**
	 * Get all available page templates
	 *
	 * @since    1.0.0
	 */
	public function get_page_template() {

		$templates = get_page_templates();

		$return = array( 'default' => esc_html__( 'Default', 'adsanity-caa' ) );

		foreach ( $templates as $key => $value ) {
			$return[ $value ] = $key;
		}

		wp_send_json_success( $return );

	}

	/**
	 * Get all available post types
	 *
	 * @since    1.0.0
	 */
	public function get_post_type_archive() {

		$this->get_post_type();

	}

	/**
	 * Endpoint for dismissing plugin compatibility
	 *
	 * @since    1.0.0
	 */
	public function dismiss_plugin_compatibility() {

		update_option( $this->plugin_name . '_plugin_compatibility_dismissed', true );

	}

}
