<?php

/**
 * The admin-specific functionality of the plugin.
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/admin
 */

/**
 * The admin-specific functionality of the plugin.
 *
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/admin
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_Caa_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

	}

	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles() {

		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/adsanity-caa-admin.css', array(), $this->version, 'all' );

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts() {

		wp_enqueue_script(
			$this->plugin_name,
			plugin_dir_url( __FILE__ ) . 'js/adsanity-caa-admin.js',
			array( 'jquery', 'jquery-ui-autocomplete' ),
			$this->version,
			true
		);

	}

	/**
	 * Add additional metabox
	 *
	 * @since    1.0.0
	 */
	public function add_meta_box() {

		global $current_screen;

		if ( ! is_admin() ) {
			return;
		}

		add_meta_box(
			'adsanity-caa',                                    // id
			__( 'Conditional Ad Appearance', 'adsanity-caa' ), // title
			array( $this, 'meta_box_cb' ),                      // callback
			'ads',                                             // screen
			'normal',                                          // context
			'low'                                              // priority
		);

	}

	/**
	 * Callback for the metabox
	 *
	 * @since    1.0.0
	 */
	public function meta_box_cb() {

		require_once plugin_dir_path( __FILE__ ) . 'partials/metabox.php';

	}

	/**
	 * Save post meta
	 *
	 * @since    1.0.0
	 */
	public function save_post( $post_id ) {

		if ( ! is_admin() ) {
			return $post_id;
		}

		if ( ! isset( $_POST['adsanity_caa'] ) ) {
			return $post_id;
		}

		if ( ! wp_verify_nonce( $_POST['adsanity_caa'], 'adsanity_caa' ) ) {
			return $post_id;
		}

		if ( ! isset( $_POST['caa-enabled'] ) ) {
			$this->enable( false, $post_id );
			return $post_id;
		} else {
			$this->enable( true, $post_id );
		}

		if ( ! isset( $_POST['caa-rules'] ) ) {
			return $post_id;
		}

		$dirty = $_POST['caa-rules'];

		$clean = array();

		// Each array is an 'or' condition ruleset
		foreach ( $dirty as $key => $value ) {

			// Each array is a 'and' condition row
			foreach ( $value as $index => $row ) {


				$clean_row = array();

				// Each item is a select box
				foreach ( $row as $name => $value ) {

					$clean_row[ sanitize_text_field( $name ) ] = sanitize_text_field( $value );

				}

				$clean[ $key ][ $index ] = $clean_row;

			}

		}

		update_post_meta( $post_id, $this->plugin_name . '-rules', $clean );

	}

	/**
	 * Add or remove the enabled meta information
	 *
	 * @since    1.0.0
	 */
	private function enable( $bool, $post_id ) {

		$meta_key = $this->plugin_name . '-enabled';

		if ( $bool ) {
			update_post_meta( $post_id, $meta_key, 1 );
			return;
		}

		delete_post_meta( $post_id, $meta_key, 1 );

	}

	/**
	 * Add an admin notice if this plugin is installed along with User Role Ad Visibility
	 *
	 * @since    1.0.0
	 */
	public function plugin_compatibility_notice() {

		if( ! function_exists( 'run_adsanity_hapr' ) ) {
			return;
		}

		$dismissed = get_option( $this->plugin_name . '_plugin_compatibility_dismissed', false );

		if ( ! $dismissed ) {

			printf( '<div class="adsanity-caa-plugin-compatibility notice notice-warning is-dismissible"><p>%s</p></div>',
				esc_html__( 'We\'ve detected that you\'re using both the User Role Ad Visibility add-on and the Conditional Ad Display add-on for AdSanity. The User Role Ad Visibility settings will override the Conditional Ad Display settings.', 'adsanity-caa' )
			);

		}

	}

	/**
	 * Adds the changelog to AdSanity's main changelog page.
	 *
	 * @since 1.3
	 */
	public static function changelog() {
		require_once plugin_dir_path( __FILE__ ) . 'partials/changelog.php';
	}

}
