<?php

/**
 * Class responsible for outputting the metabox based on rules
 */
class AdSanity_CAA_Metabox {

	/**
	 * The conditions array
	 */
	private static $conditions;

	/**
	 * The post meta enabled
	 */
	private static $enabled;

	/**
	 * The post meta for rules
	 */
	private static $rules;

	/**
	 * Initialize the class
	 */
	public static function init() {

		// Initialize the conditions array
		self::$conditions = array(
			'posts' => array(
				'optgroup_label'         => __( 'Posts', 'adsanity-caa' ),
				'autocomplete_post_post' => __( 'Post', 'adsanity-caa' ),
				'post_format'            => __( 'Post Format', 'adsanity-caa' ),
			),
			'pages' => array(
				'optgroup_label'                => __( 'Pages', 'adsanity-caa' ),
				'autocomplete_post_page'        => __( 'Page', 'adsanity-caa' ),
				'page_type'                     => __( 'Page Type', 'adsanity-caa' ),
				'page_template'                 => __( 'Page Template', 'adsanity-caa' ),
				'autocomplete_post_page_parent' => __( 'Page Parent', 'adsanity-caa' ),
			),
			'user' => array(
				'optgroup_label'    => __( 'Users', 'adsanity-caa' ),
				'autocomplete_user' => __( 'User', 'adsanity-caa' ),
				'user_type'         => __( 'User Type', 'adsanity-caa' ),
			),
			'cpt' => array(
				'optgroup_label'     => __( 'Post Types', 'adsanity-caa' ),
				'post_type'          => __( 'Post Type', 'adsanity-caa' ),
				'post_type_archive'  => __( 'Post Type Archive', 'adsanity-caa' ),
			),
			'taxonomy' => array(
				'optgroup_label'                => __( 'Taxonomies', 'adsanity-caa' ),
				'autocomplete_taxonomy_post'    => __( 'Post Taxonomy', 'adsanity-caa' ),
				'autocomplete_taxonomy_archive' => __( 'Taxonomy Archive', 'adsanity-caa' ),
			),
		);

		// Get the post meta
		self::get_meta();

		// Add post types to conditions
		self::get_post_types();

		// Output the metabox
		self::output();

	}

	/**
	 * Get the post meta
	 */
	private static function get_meta() {

		// Get the post ID
		global $post;
		$post_id = $post->ID;

		// Check to see if caa is enabled
		self::$enabled = get_post_meta( $post_id, 'adsanity-caa-enabled', true );

		// Get the rules
		self::$rules = get_post_meta( $post_id, 'adsanity-caa-rules', true );

		if ( ! self::$rules ) {
			self::$rules = array();
		}

	}

	/**
	 * Add post types to conditions
	 */
	private static function get_post_types() {

		// Get all of the public post types
		$post_types = get_post_types( array(
			'public' => true,
		) );

		// Exclude ads, posts, and pages
		unset( $post_types[ 'ads' ] );
		unset( $post_types[ 'post' ] );
		unset( $post_types[ 'page' ] );

		// Add to the conditions array
		foreach ( $post_types as $key => $val ) {
			$name = get_post_type_object( $key )->labels->name;
			self::$conditions['cpt'][ 'autocomplete_post_' . $key ] = $name;
		}

	}

	/**
	 * Output the metabox
	 *
	 * This is the only method that prints or echos
	 */
	private static function output() {

		// Output the nonce field
		wp_nonce_field( 'adsanity_caa', 'adsanity_caa' );

		// Output the enabled metabox
		self::enabled_checkbox();

		printf( '<h4>%s:</h4>',
			esc_html__( 'Show ad if', 'adsanity-caa' )
		);

		if ( ! self::$rules ) {
			echo self::display_ruleset();
		} else {

			// Loop over each ruleset
			foreach ( self::$rules as $ruleset ) {
				echo self::display_ruleset( $ruleset );
			}

		}

	}

	/**
	 * The enabled metabox
	 */
	private static function enabled_checkbox() {

		$format = '<label><b>%s: </b>
			<input id="adsanity-caa-switch" type="checkbox" name="caa-enabled" %s/>
		</label>';

		printf( $format,
			esc_html__( 'Enable', 'adsanity-caa' ),
			self::$enabled ? 'checked="checked"' : ''
		);

	}

	/**
	 * Display a ruleset
	 */
	private static function display_ruleset( $ruleset = array( null ) ) {

		ob_start();

		echo '<div class="ruleset">';

		foreach ( $ruleset as $row ) {

			echo self::display_row( $row );

		}

		echo '</div>';

		// Get the index of this rulset in $rules
		$ruleset_index = array_search( $ruleset, self::$rules );

		printf( '<div class="or %s">%s</div>',
			( $ruleset_index === count( self::$rules ) - 1 ) || ! self::$rules ? 'button' : '',
			esc_html__( 'Or', 'adsanity-caa' )
		);

		return ob_get_clean();

	}

	/**
	 * Display a single row
	 */
	private static function display_row( $row ) {

		$condition = self::display_condition( $row );
		$bool = self::display_bool( $row );
		$data = self::display_data( $row );

		$row_format = '<div class="row">
			%s
			%s
			%s
			<div class="button remove">%s</div>
			<div class="button and">%s</div>
		</div>';

		return sprintf( $row_format,
			$condition,
			$bool,
			$data,
			esc_html__( 'Remove', 'adsanity-caa' ),
			esc_html__( 'And', 'adsanity-caa' )
		);

	}

	/**
	 * Display the condition select box
	 */
	private static function display_condition( $row ) {

		if ( isset( $row['condition'] ) ) {
			$condition = $row['condition'];
		} else {
			$condition = null;
		}

		ob_start();

		echo '<select class="condition">';

			foreach ( self::$conditions as $optgroup ) {
				printf( '<optgroup label="%s">',
					esc_attr( $optgroup['optgroup_label'] )
				);

				foreach ( $optgroup as $key => $value ) {

					if ( $key === 'optgroup_label' ) {
						continue;
					}

					printf( '<option value="%s" %s>%s</option>',
						esc_attr( $key ),
						selected( $condition === $key, true, false ),
						esc_html( $value )
					);

				}

				echo '</optgroup>';
			}

		echo '</select>';

		return ob_get_clean();

	}

	/**
	 * Display the bool select box
	 */
	private static function display_bool( $row ) {

		if ( isset( $row['bool'] ) ) {
			$bool = $row['bool'];
		} else {
			$bool = 'equal';
		}

		$bool_format = '<select class="bool">
			<option value="equal" %s>' . esc_html__( 'is equal to', 'adsanity-caa' ) . '</option>
			<option value="not-equal" %s>' . esc_html__( 'is not equal to', 'adsanity-caa' ) . '</option>
		</select>';

		return sprintf( $bool_format,
			selected( $bool === 'equal', true, false ),
			selected( $bool === 'equal', false, false )
		);

	}

	/**
	 * Display data
	 */
	private static function display_data( $row ) {

		if ( isset( $row['data'] ) ) {
			$data = $row['data'];
		} else {
			$data = '';
		}

		if ( isset( $row['condition'] ) ) {
			$condition = $row['condition'];
		} else {
			$condition = '';
		}

		$data_format = '<div class="data" data-value="%1$s">
			<select></select>
			<input type="text" value="%2$s">
			<input type="hidden" value="%3$s">
		</div>
		<div class="spinner" data-value="%1$s"></div>';

		$input_text = '';
		$input_hidden = '';

		// Set the id as the hidden value if this is an autocomplete field
		if ( substr( $condition, 0, 12 ) === 'autocomplete' ) {
			$input_hidden = $row['data'];
		}

		// Get the title if this is an autocomplete post field
		if ( substr( $condition, 0, 17 ) === 'autocomplete_post' ) {
			$input_text = get_the_title( $input_hidden );
		}

		// Get the user nicename if this is an autocomplete user field
		if ( substr( $condition, 0, 17 ) === 'autocomplete_user' ) {
			$user = get_userdata( $input_hidden );
			$input_text = $user->user_nicename;
		}

		// Get the term name if this is an autocomplete taxonomy field
		if ( substr( $condition, 0, 21 ) === 'autocomplete_taxonomy' ) {
			global $wp_taxonomies;
			$term = get_term( explode( ',', $input_hidden )[0] );
			$input_text = esc_html( $term->name );

			if ( isset( $wp_taxonomies[ $term->taxonomy ] ) ) {
				$input_text .=  sprintf( ' - %s ',
					esc_html( $wp_taxonomies[ $term->taxonomy ]->label )
				);

				$post_type_names = array();

				foreach ( $wp_taxonomies[ $term->taxonomy ]->object_type as $post_type ) {
					array_push( $post_type_names, get_post_type_object( $post_type )->labels->name );
				}

				$input_text .= sprintf( '(%s) ',
					implode( ', ', $post_type_names )
				);
			}
		}

		return sprintf( $data_format,
			esc_attr( $data ),
			esc_attr( $input_text ),
			esc_attr( $input_hidden )
		);

	}

}

AdSanity_CAA_Metabox::init();
