<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/includes
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_Caa {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Adsanity_Caa_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {

		$this->plugin_name = 'adsanity-caa';
		$this->version = '1.0.0';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_admin_ajax_hooks();
		$this->define_public_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - Adsanity_Caa_Loader. Orchestrates the hooks of the plugin.
	 * - Adsanity_Caa_i18n. Defines internationalization functionality.
	 * - Adsanity_Caa_Admin. Defines all hooks for the admin area.
	 * - Adsanity_Caa_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-adsanity-caa-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-adsanity-caa-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-adsanity-caa-admin.php';

		/**
		 * The class responsible for defining all actions related to admin ajax.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-adsanity-caa-admin-ajax.php';

		/**
		 * The class responsible for defining all actions that occur in the public-facing
		 * side of the site.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-adsanity-caa-public.php';

		/**
		 * Update engine
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'update-engine/remote-api.php';

		$this->loader = new Adsanity_Caa_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the Adsanity_Caa_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new Adsanity_Caa_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new Adsanity_Caa_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
		$this->loader->add_action( 'add_meta_boxes_ads', $plugin_admin, 'add_meta_box' );
		$this->loader->add_action( 'save_post', $plugin_admin, 'save_post', 10, 1 );
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'plugin_compatibility_notice' );
		$this->loader->add_action( 'adsanity_changelog_screen', $plugin_admin, 'changelog' );

	}

	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_ajax_hooks() {

		$plugin_admin_ajax = new Adsanity_Caa_Admin_Ajax( $this->get_plugin_name(), $this->get_version() );
		$ajax_prefix = 'wp_ajax_' . $this->plugin_name . '_';

		$this->loader->add_action( $ajax_prefix . 'autocomplete_post', $plugin_admin_ajax, 'autocomplete_post' );
		$this->loader->add_action( $ajax_prefix . 'autocomplete_user', $plugin_admin_ajax, 'autocomplete_user' );
		$this->loader->add_action( $ajax_prefix . 'autocomplete_term', $plugin_admin_ajax, 'autocomplete_term' );
		$this->loader->add_action( $ajax_prefix . 'get_post_type', $plugin_admin_ajax, 'get_post_type' );
		$this->loader->add_action( $ajax_prefix . 'get_user_type', $plugin_admin_ajax, 'get_user_type' );
		$this->loader->add_action( $ajax_prefix . 'get_page', $plugin_admin_ajax, 'get_page' );
		$this->loader->add_action( $ajax_prefix . 'get_page_type', $plugin_admin_ajax, 'get_page_type' );
		$this->loader->add_action( $ajax_prefix . 'get_page_template', $plugin_admin_ajax, 'get_page_template' );
		$this->loader->add_action( $ajax_prefix . 'get_attachment', $plugin_admin_ajax, 'get_attachment' );
		$this->loader->add_action( $ajax_prefix . 'get_post_format', $plugin_admin_ajax, 'get_post_format' );
		$this->loader->add_action( $ajax_prefix . 'get_post_type_archive', $plugin_admin_ajax, 'get_post_type_archive' );
		$this->loader->add_action( $ajax_prefix . 'dismiss_plugin_compatibility', $plugin_admin_ajax, 'dismiss_plugin_compatibility' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new Adsanity_Caa_Public( $this->get_plugin_name(), $this->get_version() );

		// Priority of 10, Hide Ads Per Role is 100
		$this->loader->add_filter( 'adsanity_hide_ad', $plugin_public, 'hide_ad', 10, 2 );
		$this->loader->add_filter( 'adsanity_hide_ad_in_group', $plugin_public, 'hide_ad_in_group', 10, 2 );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Adsanity_Caa_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
