<?php

/**
 * Methods to determine whether to show or hide an ad
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/public
 */

/**
 * Methods to determine whether to show or hide an ad
 *
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/public
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_Caa_Conditional_Logic {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {}

	/**
	 * Check if the post id matches
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    int    $post_id   The page id
	 * @param    string    $condition_name    The name of the condition
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function autocomplete_post( $condition, $post_id, $condition_name ) {

		// Special condition for posts page
		if ( is_home() ) {
			if ( intval( get_option( 'page_for_posts' ) ) === intval( $post_id ) ){
				return ! $condition;
			}
			return $condition;
		}

		// is_single does not work on page and attachment
		if ( $condition_name === 'autocomplete_post_page' ||
			 $condition_name === 'autocomplete_post_attachment' ) {

			if ( intval( get_the_ID() ) === intval( $post_id ) ) {
				return ! $condition;
			}

			return $condition;

		} elseif ( $condition_name === 'autocomplete_post_page_parent' ) {

			return $this->page_parent( $condition, $post_id );

		} else {

			if ( is_single( $post_id ) ) {
				return ! $condition;
			}

			return $condition;

		}

	}

	/**
	 * Check if this user matches
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    int    $user_id    The user id
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function autocomplete_user( $condition, $user_id ) {

		if ( intval( get_current_user_id() ) === intval( $user_id ) ) {
			return ! $condition;
		}

		return $condition;

	}

	/**
	 * Check if post term matches
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    string    $term_info    term id and taxonomy ex: '123,category'
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function autocomplete_taxonomy_post( $condition, $term_info ) {

		$term_id = explode( ',', $term_info )[0];
		$taxonomy = explode( ',', $term_info )[1];

		if ( ! is_singular() ) {
			return $condition;
		}

		if ( $taxonomy === 'category' ) {

			if ( in_category( intval( $term_id ) ) ) {
				return ! $condition;
			}

		} else if ( $taxonomy === 'post_tag' ) {

			if ( has_tag( intval( $term_id ) ) ) {
				return ! $condition;
			}

		} else {

			if ( has_term( intval( $term_id ), $taxonomy ) ) {
				return ! $condition;
			}

		}

		return $condition;

	}

	/**
	 * Check if this is a taxonomy archive
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    string    $term_info    term id and taxonomy ex: '123,category'
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function autocomplete_taxonomy_archive( $condition, $term_info ) {

		$term_id = explode( ',', $term_info )[0];
		$taxonomy = explode( ',', $term_info )[1];

		if ( is_singular() ) {
			return $condition;
		}

		if ( $taxonomy === 'category' ) {

			if ( is_category( intval( $term_id ) ) ) {
				return ! $condition;
			}

		} else if ( $taxonomy === 'post_tag' ) {

			if ( is_tag( intval( $term_id ) ) ) {
				return ! $condition;
			}

		} else {

			if ( is_tax( $taxonomy, intval( $term_id ) ) ) {
				return ! $condition;
			}

		}

		return $condition;

	}

	/**
	 * Check if this post type matches
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    string    $post_type    The post type
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function post_type( $condition, $post_type ) {

		if ( is_singular( $post_type ) ) {
			return ! $condition;
		}

		return $condition;

	}

	/**
	 * Check if this user type matches
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    string    $user_type    The user type
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function user_type( $condition, $user_type ) {

		$user = wp_get_current_user();

		if ( current_user_can( $user_type ) ) {
			return ! $condition;
		}

		return $condition;

	}

	/**
	 * Check if the post format matches
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    string    $post_format    The post format
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function post_format( $condition, $post_format ) {

		$format = get_post_format();

		if ( $format == $post_format ) {
			return ! $condition;
		}

		return $condition;

	}

	/**
	 * Check if the page type matches
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    string    $page_type    The ppage type
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function page_type( $condition, $page_type ) {

		global $post;

		switch ( $page_type ) {
			case 'front_page' :
				return is_front_page() ? ! $condition : $condition;
			case 'posts_page' :
				return is_home() ? ! $condition : $condition;
			case 'top_level' :
				return $post->post_parent == 0 ? ! $condition : $condition;
			case 'parent'     :
				$children = get_pages( array( 'child_of' => $post->ID ) );
				return count( $children ) > 0 ? ! $condition : $condition;
			case 'child' :
				return $post->post_parent != 0 ? ! $condition : $condition;
		}

		return $condition;

	}

	/**
	 * Check if the page parent id matches this page's parent
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    int    $page_id    The parent page id
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function page_parent( $condition, $parent_id ) {

		global $post;

		if ( intval( $post->post_parent ) === intval( $parent_id ) ) {
			return ! $condition;
		}

		return $condition;

	}

	/**
	 * Check if the page template matches
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    string    $page_template    The page template
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function page_template( $condition, $page_template ) {

		global $template;

		if ( basename( $template ) == $page_template ) {
			return ! $condition;
		}

		$default_templates = apply_filters( 'adsanity_caa_default_page_templates', array(
			'page.php',
			'index.php',
		) );

		if ( $page_template == 'default' &&
			 is_page() &&
			 in_array( basename( $template ), $default_templates ) ) {
			return ! $condition;
		}

		return $condition;

	}

	/**
	 * Check if this post type matches
	 *
	 * @since    1.0.0
	 * @param    bool    $condition    True for equal, false for not equal
	 * @param    string    $post_type    The post type
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function post_type_archive( $condition, $post_type ) {

		if ( $post_type === 'post' ) {
			if ( is_home() ) {
				return ! $condition;
			}
		}

		if ( is_post_type_archive( $post_type ) ) {
			return ! $condition;
		}

		return $condition;

	}

}
