<?php

/**
 * The public-facing functionality of the plugin.
 *
 * @link       https://www.pixeljar.com
 * @since      1.0.0
 *
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/public
 */

/**
 * The public-facing functionality of the plugin.
 *
 * @package    Adsanity_Caa
 * @subpackage Adsanity_Caa/public
 * @author     Pixel Jar <info@pixeljar.com>
 */
class Adsanity_Caa_Public {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The conditional logic class.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      class    $conditional_logic    The current version of this plugin.
	 */
	private $conditional_logic;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

		$this->load_dependencies();
		$this->conditional_logic = new Adsanity_Caa_Conditional_Logic();

	}

	/**
	 * Load dependencies for this class
	 *
	 * @since    1.0.0
	 */
	private function load_dependencies() {

		/**
		 * Require the conditional logic class
		 */
		require_once plugin_dir_path( __FILE__ ) . 'class-adsanity-caa-conditional-logic.php';

	}

	/**
	 * Determing whether to hide or show ad
	 *
	 * @since    1.0.0
	 * @param    bool    $bool    The boolean for the ad
	 * @param    integer    $ad_id    An ad id
	 * @return    bool    Return true if hiding ad, false if showing
	 */
	public function hide_ad( $bool, $ad_id ) {

		// Check to see if conditional logic is enabled
		$enabled = get_post_meta( $ad_id, 'adsanity-caa-enabled', true );

		// If not enabled exist, return
		if ( ! $enabled || ( defined( 'REST_REQUEST' ) && REST_REQUEST ) ) {
			return $bool;
		}

		// Get the conditional logic rules
		$rules = get_post_meta( $ad_id, 'adsanity-caa-rules', true );

		// If no rules exist, return
		if ( ! $rules ) {
			return $bool;
		}


		/**
		 * Create an array with the results
		 */
		$results = array();

		// Each array is a ruleset
		foreach ( $rules as $key => $ruleset ) {

			// Create new array in results for this ruleset
			$results[ $key ] = array();

			// Each array is a row
			foreach ( $ruleset as $row ) {

				// Turn condition into a boolean
				$condition = $row['bool'] == 'equal' ? true : false;

				// Call a conditional logic class method which will return a boolean
				if ( substr( $row['condition'], 0, 17 ) === 'autocomplete_post' ) {

					// If it is an autocomplete post, use that method
					$row_bool = $this->conditional_logic->autocomplete_post( $condition, $row['data'], $row['condition'] );
					array_push( $results[ $key ], $row_bool );

				} elseif ( substr( $row['condition'], 0, 17 ) === 'autocomplete_user' ) {

					// If it is an autocomplete post, use that method
					$row_bool = $this->conditional_logic->autocomplete_user( $condition, $row['data'] );
					array_push( $results[ $key ], $row_bool );

				} else {

					$row_bool = $this->conditional_logic->{ $row['condition'] }( $condition, $row['data'] );
					array_push( $results[ $key ], $row_bool );

				}

			}

		}

		// Loop over each ruleset result
		foreach ( $results as $ruleset ) {

			// Set ruleset to false by default (show ad)
			$ruleset_bool = false;

			// Loop over each row
			foreach ( $ruleset as $row ) {

				// If it ever evaluates to true, it is true (hide ad)
				if ( $row ) {
					$ruleset_bool = true;
					break;
				}
			}

			// If a ruleset returns false, this rule evaluates to false (show ad)
			if ( ! $ruleset_bool ) {
				return false;
			}

		}

		// Finally, return true if everything evaluated to true (hide ad)
		return true;

	}

	/**
	 * Return an array of banned post ids from groups
	 * @since    1.0.0
	 * @param    array    $banned    Incoming array of banned ids
	 * @param    array    $group_ids    An ad id
	 * @return    array    Array of banned ids 
	 */
	public function hide_ad_in_group( $banned, $group_ids ) {

		$now = time();

		$ads = get_posts( array(
			'post_type' => 'ads',
			'fields'    => 'ids',
			'tax_query' => array(
				array(
					'taxonomy' => 'ad-group',
					'field'    => 'id',
					'terms'    => implode( ',', $group_ids )
				)
			),
			'meta_query' => array(
				array(
					'key'     => '_start_date',
					'value'   => $now,
					'type'    => 'numeric',
					'compare' => '<='
				),
				array(
					'key'     => '_end_date',
					'value'   => $now,
					'type'    => 'numeric',
					'compare' => '>='
				)
			),
			'posts_per_page' => -1,
			'no_found_rows'  => true,
		) );

		foreach ( $ads as $ad ) {
			if ( $this->hide_ad( false, $ad ) ) {
				array_push( $banned, $ad );
			}
		}

		return $banned;

	}

}
