<?php

/**
 * Ajax endpoints for the admin area
 */

class Adsanity_GATI_Admin_Ajax {

	/**
	 * Unique identifier
	 */
	private $plugin_name;

	/**
	 * The version for this plugin
	 */
	private $version;

	/**
	 * Reference to the oAuth class
	 */
	private $oauth;

	/**
	 * Initialize the class and set its properties.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;

		$this->load_dependencies();
		$this->oauth = new Adsanity_GATI_Admin_oAuth( $this->plugin_name, $this->version );
	}

	/**
	 * Load dependencies for this class
	 */
	private function load_dependencies() {

		/**
		 * Require the oAuth class
		 */
		require_once( plugin_dir_path( __FILE__ ) . 'class-adsanity-gati-admin-ajax.php' );

	}

	/**
	 * Admin ajax endpoint for authenticating the Google authorization code
	 */
	public function authenticate_code() {
		$code = $_POST['authorization_code'];

		$this->oauth->exchange_for_tokens( $code );
	}

	/**
	 * Admin ajax endpoint for getting analytics profiles
	 */
	public function get_profiles() {
		$access_token = adsanity_gati_get_access_token();

		$args = array(
			'body' => array(
				'access_token' 	=> $access_token,
			),
		);

		$response = wp_remote_get( 'https://www.googleapis.com/analytics/v3/management/accountSummaries', $args );

		if ( is_wp_error( $response ) ) {
			wp_send_json_error( $response->get_error_message(), 400 );
		}

		if ( ! array_key_exists( 'body', $response ) ) {
			wp_send_json_error( __( 'No content found in the account summaries reponse.', 'adsanity-gati' ), 400 );
		}

		$body = json_decode( wp_remote_retrieve_body( $response ) );

		if ( ! property_exists( $body, 'items' ) ) {
			wp_send_json_error( __( 'No items found in the content body of the account summaries.', 'adsanity-gati' ), 400 );
		}

		wp_send_json_success( $body->items );
	}

	/**
	 * Admin ajax endpoint for removing authentication tokens from site
	 */
	public function remove_authentication() {

		delete_option( $this->plugin_name . '-already_tracking' );
		delete_option( $this->plugin_name . '-refresh_token' );
		delete_option( $this->plugin_name . '-profile_id' );
		delete_option( $this->plugin_name . '-tracking_id' );
		wp_clear_scheduled_hook( 'adsanity_gati_get_access_token' );

		wp_send_json_success();
	}

	/**
	 * Ajax endpoint to set profile
	 */
	public function set_profile() {
		$id = $_POST[ 'id' ];
		$tracking = $_POST[ 'tracking' ];

		update_option( $this->plugin_name . '-profile_id', $id );
		update_option( $this->plugin_name . '-tracking_id', $tracking );

		wp_send_json_success();
	}

	/**
	 * Ajax endpoint to retrieve custom report
	 */
	public function custom_reports() {
		$access_token = adsanity_gati_get_access_token();
		$profile_id = 'ga:' . get_option( $this->plugin_name . '-profile_id' );

		if ( ! $_POST[ 'startDate' ] || ! $_POST[ 'endDate' ] || ! $_POST[ 'ads' ] ) {
			wp_send_json_error( __( 'Missing parameters from the search query.', 'adsanity-gati' ), 400 );
		}

		$start_date = $_POST[ 'startDate' ];
		$end_date = $_POST[ 'endDate' ];
		$ads = $_POST[ 'ads' ];

		$num_of_ads = count( AdSanityQuery::get_all_ads() );

		$ad_string = '';

		// If not all ads selected, construct filter param
		$construct_filter = apply_filters( 'adsanity_gati_construct_custom_reports_filter', count( $ads ) !== $num_of_ads );

		if ( $construct_filter ) {

			// Add to the filters param
			foreach ( $ads as $key => $val ) {
				if ( $key === 0 ) {
					$ad_string .= ';ga:eventLabel==' . $val;
				} else {
					$ad_string .= ',ga:eventLabel==' . $val;
				}
			}
		}

		$args = array(
			'method' => 'GET',
			'body' => array(
				'start-date' 	=> $start_date,
				'end-date' 		=> $end_date,
				'ids' 			=> $profile_id,
				'metrics' 		=> 'ga:totalEvents',
				'dimensions' 	=> 'ga:eventCategory,ga:eventAction,ga:eventLabel,ga:date',
				'filters' 		=> 'ga:eventCategory==AdSanity' . $ad_string,
				'access_token' 	=> $access_token,
				'sort'			=> 'ga:date'
			),
		);

		$response = wp_remote_get( 'https://www.googleapis.com/analytics/v3/data/ga', $args );

		if ( is_wp_error( $response ) ) {
			wp_send_json_error( $response->get_error_message(), 400 );
		}

		if ( ! array_key_exists( 'body', $response ) ) {
			wp_send_json_error( __( 'No content found in the data reponse.', 'adsanity-gati' ), 400 );
		}

		$body = json_decode( wp_remote_retrieve_body( $response ) );

		if ( ! property_exists( $body, 'rows' ) ) {
			wp_send_json_error( __( 'No rows found in the content body of the data response.', 'adsanity-gati' ), 400 );
		}

		$report = array();

		// Construct the report
		foreach ( $body->rows as $row ) {
			$date = date( 'F j, Y', strtotime( $row[ 3 ] ) );
			$report[ $date ][ $row[ 2 ] ][ $row[ 1 ] ] = $row[ 4 ];
		}

		// Add CTR
		foreach ( $report as $key => $value ) {

			foreach ( $value as $inner_key => $inner_value ) {

				if ( array_key_exists( 'click', $inner_value ) ) {
					$ctr = round( $inner_value[ 'click' ] / $inner_value[ 'view' ] * 100, 2 );
					$report[ $key ][ $inner_key ][ 'ctr' ] = $ctr;
				}
			}
		}

		wp_send_json_success( $report );
	}

	/**
	 * Endpoint for detection of MonsterInsights
	 */
	public function using_monster_insights() {
		if ( function_exists( 'MonsterInsights' ) ) {
			// Create an option we can use later for admin notice
			update_option( $this->plugin_name . '-using_monster_insights', true );

			update_option( $this->plugin_name . '-already_tracking', 'true' );
			wp_send_json_success( array( 'usingMonster' => true ) );
		}

		delete_option( $this->plugin_name . '-using_monster_insights' );

		wp_send_json_success( array( 'usingMonster' => false ) );
	}

	/**
	 * Endpoint for setting whether user is using Google Analytics already
	 */
	public function using_google_analytics() {
		$already_tracking = $_POST[ 'alreadyTracking' ];
		$updated = update_option( $this->plugin_name . '-already_tracking', $already_tracking );
		if ( ! $updated ) {
			wp_send_json_error( __( 'The option could not be updated.', 'adsanity-gati' ), 400 );
		}

		wp_send_json_success();
	}

	/**
	 * Endpoint for setting tracking Id manually
	 */
	public function manual_tracking_id() {
		$tracking_id = $_POST[ 'trackingId' ];
		update_option( $this->plugin_name . '-tracking_id', $tracking_id );
		wp_send_json_success();
	}

	/**
	 * Endpoint for dismissing the expectations warning
	 */
	public function dismiss_expectation() {
		update_option( $this->plugin_name . '-expectations_dismissed', true );
	}

	/**
	 * Get the ads and stats based on term_id
	 */
	public function get_ads_by_term() {

		if ( ! isset( $_POST['term_id'] ) ) {
			return;
		}

		global $wp_locale;

		$term_id = intval( $_POST['term_id'] );

		// // Get ads based on term id
		$ad_args = array(
			'post_type'      => 'ads',
			'posts_per_page' => -1,
			'tax_query'      => array(
				array(
					'taxonomy' => 'ad-group',
					'field'    => 'term_id',
					'terms'    => $term_id,
				),
			),
		);

		$ads = new WP_Query( $ad_args );

		$access_token = adsanity_gati_get_access_token();
		$profile_id = 'ga:' . get_option( $this->plugin_name . '-profile_id' );
		$ads_in_group = array();

		// Create an array of ad titles for Google query
		if ( $ads->have_posts() ) {
			while ( $ads->have_posts() ) {
				$ads->the_post();
				array_push( $ads_in_group, get_the_title() );
			}
		}

		$filter = 'ga:eventCategory==AdSanity';

		foreach ( $ads_in_group as $key => $ad ) {
			if ( $key === 0 ) {
				$filter .= ';ga:eventLabel==' . $ad;
			} else {
				$filter .= ',ga:eventLabel==' . $ad;
			}
		}

		$args = array(
			'method' 	=> 'GET',
			'body' 		=> array(
				'start-date' 	=> '2005-01-01',
				'end-date' 		=> 'today',
				'ids' 			=> $profile_id,
				'metrics' 		=> 'ga:totalEvents',
				'dimensions' 	=> 'ga:eventCategory,ga:eventAction,ga:eventLabel',
				'filters' 		=> $filter,
				'access_token' 	=> $access_token,
			),
		);

		$response = wp_remote_get( 'https://www.googleapis.com/analytics/v3/data/ga', $args );

		if ( is_wp_error( $response ) ) {
			wp_send_json_error( $response->get_error_message(), 400 );
		}

		if ( ! array_key_exists( 'body', $response ) ) {
			wp_send_json_error( __( 'No content found in the terms response body.', 'adsanity-gati' ), 400 );
		}

		$body_json = json_decode( wp_remote_retrieve_body( $response ) );

		$results = array();

		if ( property_exists( $body_json, 'rows' ) ) {

			// // Loop through and construct results array from Google results
			foreach ( $body_json->rows as $row ) {
				$results[ $row[2] ][ $row[1] ] = $row[3];
			}

			// // Loop through results and add ctr
			foreach( $results as $key => $result ) {
				if ( ! array_key_exists( 'click', $results[ $key ] ) ) {
					$results[ $key ]['click'] = 0;
				}

				if ( ! array_key_exists( 'view', $results[ $key ] ) ) {
					$results[ $key ]['view'] = 0;
					continue;
				}

				$results[ $key ]['ctr'] = $results[ $key ]['click'] / $results[ $key ]['view'] * 100;
			}

			// Sort results by ctr
			uasort( $results, array( $this, 'sort' ) );

			// Change format of ctr
			foreach ( $results as $key => $val ) {
				$results[ $key ]['ctr'] = number_format(
					$results[ $key ]['ctr'],
					2,
					$wp_locale->number_format['decimal_point'],
					$wp_locale->number_format['thousands_sep']
				) . '%';
			}
		}

		wp_send_json_success( $results );

	}

	/**
	 * Sort results by ctr
	 */
	public function sort( $a, $b ) {
		if ( $a["ctr"] == $b["ctr"] ) {
			return 0;
		}
		return ( $a["ctr"] > $b["ctr"] ) ? -1 : 1;
	}

}
