//
// NPM
//
import AsyncSelect from 'react-select/async';
import { AllHtmlEntities } from 'html-entities';
import arrayMove from 'array-move';
import React from 'react';

//
// WordPress imports
//
const { registerBlockType, } = window.wp.blocks;
const { Placeholder, }       = window.wp.components;
const apiFetch               = window.wp.apiFetch;
const { addQueryArgs, }      = window.wp.url;
const { __, sprintf, }       = window.wp.i18n;

// This will allow us to decode HTML entities
const entities = new AllHtmlEntities();

//
// Local imports
//
import icon from './icon';

//
// Imports from AdSanity Core
//
import AlignButtonGroup from '../../../../../adsanity/src/js/blocks/common/AlignButtonGroup';
import MaxWidthControl from '../../../../../adsanity/src/js/blocks/common/MaxWidthControl';
import AutomaticInclusion from '../../../../../adsanity/src/js/blocks/common/AutomaticInclusion';
import AdGroup from '../../../../../adsanity/src/js/blocks/common/AdGroup';
import AddOnsWarning from '../../../../../adsanity/src/js/blocks/common/AddOnsWarning';
import selectTheme from '../../../../../adsanity/src/js/blocks/common/select-theme';

//
// Local imports
//
import AdList from './AdList';

export default registerBlockType(
	'adsanity/ordered-ads',
	{
		title: sprintf(
			__( '%s Ordered Ads', 'adsanity' ),
			'AdSanity'
		),
		icon: icon,
		category: 'adsanity',
		keywords: [
			__( 'ad', 'adsanity' ),
			__( 'advertisement', 'adsanity' ),
			__( 'advertisements', 'adsanity' ),
		],
		attributes: {
			ads: {
				type: 'array',
				default: [],
			},
			currentValue: {
				type: 'text',
				default: '',
			},
			align: {
				type: 'text',
				default: 'alignnone',
			},
			max_width_enabled: {
				type: 'boolean',
				default: false,
			},
			max_width: {
				type: 'text',
				default: '100',
			},
			rendered_ad: {
				type: 'object',
				default: {
					adId: null,
					markup: '',
				},
			},
			num_columns: {
				type: 'text',
				default: '1',
			},
		},



		edit( props ) {

			//
			// Load ads for the select component
			//
			const loadOptions = ( value, callback ) => {

				let args = {
					per_page : 100,
				};

				if ( value ) {
					args.orderby = 'title';
					args.search  = value;
				}

				apiFetch( {
					path: addQueryArgs( '/wp/v2/ads/', args ),
				} )
				.then( ( posts ) => {
					posts = posts.map( ( post ) => {
						return {
							value: post.id,
							label: entities.decode( post.title.rendered ),
						};
					} );
					callback( posts );
				} )
				.catch( ( error ) => {
					console.log( error );
				} );

			}

			//
			// Changes ads in both props and state
			//
			const modifyAds = ( ads ) => {

				props.setAttributes( {
					ads: ads,
				} );

				// this.setState( {
				// 	ads: ads,
				// } );

			}

			//
			// Handles ad change of the select component
			//
			const onAdChange = ( nextValue ) => {

				const ads = props.attributes.ads.slice();
				ads.push( {
					id   : nextValue.value,
					name : nextValue.label,
				} );
				modifyAds( ads );

			}

			//
			// Handles changes of alignment buttons
			//
			const onAlignChange = ( value ) => {
				props.setAttributes( {
					align: value,
				} );
			}

			//
			// Handles changes on the Max Width Enabled checkbox
			//
			const onMaxWidthEnabledChange = () => {
				let maxWidth = '100';
				if ( props.attributes.max_width_enabled ) {
					maxWidth = '';
				}
				props.setAttributes( {
					max_width: maxWidth,
					max_width_enabled: ! props.attributes.max_width_enabled,
				} );
			}

			//
			// Handles changes on the Max Width field
			//
			const onMaxWidthChange = ( value ) => {
				props.setAttributes( {
					max_width: value,
				} );
			}

			//
			// Handles changes on the number of columns
			//
			const onNumColumnsChange = ( event ) => {
				let value = event.target.value.replace( /\D/g, '' );
				if ( value < 1 ) {
					value = '';
				}
				props.setAttributes( {
					num_columns: value,
				} );
			}

			//
			// Moves an Ad Up
			//
			const onAdUp = ( index ) => {

				// The first ad cannot be moved up
				if ( index === 0 ) {
					return;
				}
				const ads = props.attributes.ads;
				const reorderedAds = arrayMove( ads, index, index - 1 );
				modifyAds( reorderedAds );

			}

			//
			// Moves an Ad Down
			//
			const onAdDown = ( index ) => {

				const ads = props.attributes.ads;
				// The last ad cannot be moved down
				if ( index === ads.length - 1 ) {
					return;
				}
				const reorderedAds = arrayMove( ads, index, index + 1 );
				modifyAds( reorderedAds );

			}

			//
			// Removes an Ad
			//
			const onAdRemove = ( index ) => {

				const ads = props.attributes.ads;
				ads.splice( index, 1 );
				modifyAds( ads );

			}

			//
			// Handles drag end
			//
			const onDragEnd = ( source, destination ) => {

				const ads = arrayMove(
					props.attributes.ads,
					source,
					destination
				);
				modifyAds( ads );

			}

			//
			// Save a rendered ad
			//
			const onRenderedAd = ( value ) => {

				const ads = props.attributes.ads.slice();
				ads.forEach( ( ad ) => {
					if ( ad.id === value.id ) {
						ad.classes = value.classes;
						ad.isNetwork = value.isNetwork;
						ad.markup = value.markup;
					}
				} );
				modifyAds( ads );

			}

			return (
				<div
					className={ props.className }
					data-align={ props.attributes.align }
				>
					{ !! props.isSelected ? (
						<Placeholder
							label={ sprintf( __( '%s Ordered Ads', 'adsanity' ), 'AdSanity' ) }
						>
							<label>
								<span>{ __( 'Select an ad:', 'adsanity' ) }</span>
								<AsyncSelect
									cacheOptions
									defaultOptions
									loadOptions={ loadOptions }
									onChange={ onAdChange }
									theme={ selectTheme }
									value={ props.attributes.currentValue }
								/>
							</label>

							<AdList
								ads={ props.attributes.ads }
								onAdUp={ onAdUp }
								onAdDown={ onAdDown }
								onAdRemove={ onAdRemove }
								onDragEnd={ onDragEnd }
							/>

							<AddOnsWarning/>

							<hr className="adsanity-section-separator"/>

							<div className="adsanity-oagw-num-columns">
								<label>
									<span>{ __( 'Number of Columns:', 'adsanity' ) }</span>
									<input
										type="text"
										value={ props.attributes.num_columns }
										onChange={ onNumColumnsChange }
									/>
								</label>
							</div>

							<hr className="adsanity-section-separator"/>
		
							<AlignButtonGroup
								onAlignChange={ onAlignChange }
								propValue={ props.attributes.align }
							/>

							<hr className="adsanity-section-separator"/>

							<MaxWidthControl
								onMaxWidthEnabledChange={ onMaxWidthEnabledChange }
								onMaxWidthChange={ onMaxWidthChange }
								maxWidthEnabled={ props.attributes.max_width_enabled }
								maxWidth={ props.attributes.max_width }
							/>

							<AutomaticInclusion/>

						</Placeholder>
					) : (
						<div>
							<AdGroup
								align={ props.attributes.align }
								adIds={ props.attributes.ads }
								numAds={ props.attributes.ads.length }
								numColumns={ props.attributes.num_columns }
								maxWidthEnabled={ props.attributes.max_width_enabled }
								maxWidth={ props.attributes.max_width }
								onRenderedAd={ onRenderedAd }
								noAdMessage={ __( 'Please select at least one ad', 'adsanity' ) }
							/>
						</div>
					) }
				</div>
			);

		},

		save() {
			// Rendered in PHP
			return null;
		},
	}
);
