<?php

/**
 * Admin plugin functionality
 */
class Adsanity_RAW_Admin {

	/**
	 * Unique identifier
	 */
	private $plugin_name;

	/**
	 * The plugin options
	 */
	private $options;

	/**
	 * The plugin option name
	 */
	private $option_name = 'adsanity-rotating-ad-widget-options';

	/**
	 * The version for this plugin
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->set_options();

	}

	/**
	 * Register TinyMCE button for use
	 */
	public function register_tinymce_button( $buttons = array() ) {

		array_push( $buttons, 'adsanity_raw' );
		return $buttons;

	}

	/**
	 * Add button to TinyMCE
	 */
	public function add_tinymce_plugin( $plugins = array() ) {

		global $post;

		if ( isset( $post->post_type ) && 'ads' === $post->post_type ) {
			return $plugins;
		}

		$plugins['AdSanityRotatingAdWidget'] = plugin_dir_url( __FILE__ ) . 'dist/js/tinymce.js';
		return $plugins;

	}

	/**
	 * Set up translations and data for tinymce
	 *
	 * @return void
	 */
	public function tinymce_javascript_object() {

		global $current_screen;

		if ( is_admin() ) {
			?>
			<script type="text/javascript">
				var AdSanityRAWMCE = {
					adsanity_raw_title: "<?php echo esc_js( __( 'Insert Rotating Ad', 'adsanity-raw' ) ); ?>",
					adsanity_raw_num_secs_label: "<?php echo esc_js( __( 'Time (seconds)', 'adsanity-raw' ) ); ?>",
					adsanity_random_label: "<?php echo esc_js( __( 'Order Randomly?', 'adsanity-raw' ) ); ?>",
					adsanity_random_desc: "<?php echo esc_js( '(defaults to ordering by publish date)', 'adsanity-raw' ); ?>"
				};
			</script>
			<?php
		}
	}

	/**
	 * Reference to the widget class
	 */
	public function add_widget() {

		require_once( plugin_dir_path( __FILE__ ) . 'class-adsanity-raw-widget.php' );
		register_widget( 'Adsanity_RAW_Widget' );

	}

	/**
	 * Add options to AdSanity Add-Ons Settings tab
	 */
	public function add_settings_tab( $addons = array() ) {

		$addons[] = array(
			'slug' => 'rotating-ad-widget',
			'name' => __( 'Rotating Ad Widget', 'adsanity-raw' ),
		);

		return $addons;

	}

	/**
	 * Sets the class variable $options
	 */
	private function set_options() {

		$this->options = get_option( $this->option_name, array() );

	}

	/**
	 * Add error message
	 */
	private function add_error( $key, $message ) {

		add_settings_error(
			'adsanity-rotating-ad-widget-options',
			$key,
			$message,
			'error'
		);

	}

	/**
	 * Register plugin settings
	 */
	public function register_settings() {

		register_setting(
			'adsanity-rotating-ad-widget-options',
			'adsanity-rotating-ad-widget-options',
			array( $this, 'validate_options' )
		);

	}

	/**
	 * Register settings sections
	 */
	public function register_sections() {

		add_settings_section(
			$this->plugin_name . '-options',
			sprintf( __( '%s Rotating Ads', 'adsanity-raw' ), 'AdSanity' ),
			array( $this, 'section_view' ),
			'adsanity-rotating-ad-widget-options'
		);

	}

	/**
	 * Creates a settings section
	 */
	public function section_view( $params ) {

		include( plugin_dir_path( __FILE__ ) . 'partials/adsanity-rotating-ad-widget-admin-section-view.php' );

	}

	/**
	 * Register settings fields
	 */
	public function register_fields() {

		add_settings_field(
			'allow-network-ads',
			esc_html__( 'Allow Network Ads', 'adsanity-abd' ),
			array( $this, 'allow_network_ads_cb' ),
			'adsanity-rotating-ad-widget-options',
			$this->plugin_name . '-options'
		);

	}

	/**
	 * Callback for allow-network-ads field
	 */
	public function allow_network_ads_cb( $args ) {

		$allow_network_ads = false;
		$slug = 'allow-network-ads';

		if ( isset( $this->options[ $slug ] ) ) {
			$allow_network_ads = $this->options[ $slug ];
		}

		printf(
			'<label><input type="checkbox" value="1" name="%s[%s]" %s /> %s</label>',
			esc_attr( $this->option_name ),
			esc_attr( $slug ),
			checked( (bool) $allow_network_ads, true, false ),
			esc_html__( 'I understand the potential risks involved in including Network Ads in a rotating widget', 'adsanity-raw' )
		);

	}

	/**
	 * Validate options
	 */
	public function validate_options( $dirty = array() ) {

		$clean = $this->options;

		if ( isset( $dirty['allow-network-ads'] ) ) {
			$clean['allow-network-ads'] = 1;
		} else {
			$clean['allow-network-ads'] = 0;
		}

		return $clean;

	}

	/**
	 * Enqueue Gutenberg scripts
	 */
	public function gutenberg_scripts() {

		wp_enqueue_script(
			$this->plugin_name . '-block',
			plugin_dir_url( __FILE__ ) . 'dist/js/blocks.js',
			array( 'wp-blocks', 'wp-i18n', 'wp-url', 'wp-api-fetch', 'wp-element', 'wp-components', ),
			$this->version
		);

		wp_enqueue_style(
			$this->plugin_name . '-block',
			plugin_dir_url( __FILE__ ) . 'dist/css/blocks.css',
			array(),
			$this->version
		);

	}

	/**
	 * Register block type for rotating ads
	 */
	public function register_block_type() {

		if ( ! function_exists( 'register_block_type' ) ) {
			return;
		}

		register_block_type( 'adsanity/rotating-ad', array(
			'attributes' => array(
				'group_id' => array(
					'type' => 'number',
				),
				'align' => array(
					'type' => 'string',
					'default' => 'alignnone',
				),
				'time' => array(
					'type' => 'number',
					'default' => 30,
				),
				'max_width' => array(
					'type' => 'number',
					'default' => 0,
				),
				'random' => array(
					'type' => 'boolean',
					'default' => true,
				),
				// Send attribute of return => true so that the ad is not printed
				'return' => array(
					'type'    => 'boolean',
					'default' => true,
				),
			),
			'render_callback' => 'adsanity_show_rotating_ads',
		) );

	}

	/**
	 * Adds the changelog to AdSanity's main changelog page.
	 *
	 * @since 1.3
	 */
	public static function changelog() {
		require_once plugin_dir_path( __FILE__ ) . 'partials/changelog.php';
	}

}
