const { registerBlockType, } = window.wp.blocks;

//
// NPM
//
import AsyncSelect from 'react-select/lib/Async';
import { AllHtmlEntities } from 'html-entities';
import arrayShuffle from 'array-shuffle';

//
// WordPress imports
//
const {
	Placeholder,
	CheckboxControl,
	Tooltip,
	IconButton,
} = window.wp.components;
const apiFetch          = window.wp.apiFetch;
const { addQueryArgs, } = window.wp.url;
const { __, sprintf, }  = window.wp.i18n;


// This will allow us to decode HTML entities
const entities = new AllHtmlEntities();

//
// Local imports
//
import icon from './icon';

//
// Imports from AdSanity Core
//
import AlignButtonGroup from '../../../../../adsanity/src/js/blocks/common/AlignButtonGroup';
import MaxWidthControl from '../../../../../adsanity/src/js/blocks/common/MaxWidthControl';
import AutomaticInclusion from '../../../../../adsanity/src/js/blocks/common/AutomaticInclusion';
import SingleAd from '../../../../../adsanity/src/js/blocks/common/SingleAd';
import AddOnsWarning from '../../../../../adsanity/src/js/blocks/common/AddOnsWarning';
import selectTheme from '../../../../../adsanity/src/js/blocks/common/select-theme';
import colors from '../../../../../adsanity/src/js/blocks/common/colors';
import questionIcon from '../../../../../adsanity/src/js/blocks/common/questionIcon.js';

export default registerBlockType(
	'adsanity/rotating-ad',
	{
		title: sprintf(
			__( '%s Rotating Ad', 'adsanity' ),
			'AdSanity'
		),
		icon: icon,
		category: 'adsanity',
		keywords: [
			__( 'ads', 'adsanity' ),
			__( 'advertisement', 'adsanity' ),
			__( 'advertisements', 'adsanity' ),
		],
		attributes: {
			group_id: {
				type: 'number',
				default: [],
			},
			group_name: {
				type: 'text',
			},
			ad_id: {
				type: 'number',
			},
			align: {
				type: 'text',
				default: 'alignnone',
			},
			time: {
				type: 'text',
				default: '30',
			},
			max_width_enabled: {
				type: 'boolean',
				default: false,
			},
			max_width: {
				type: 'text',
				default: '100',
			},
			rendered_ad: {
				type: 'object',
				default: {
					adId: null,
					markup: '',
				},
			},
			random: {
				type: 'boolean',
				default: true,
			}
		},
		edit( props ) {

			//
			// Load ad groups for the select component
			//
			const loadOptions = ( value, callback ) => {

				let args = {
					per_page : 5,
				};

				if ( value ) {
					args.orderby = 'name';
					args.search  = value;
				}

				apiFetch( {
					path: addQueryArgs( '/wp/v2/ad-group/', args ),
				} )
				.then( ( groups ) => {
					groups = groups.map( ( group ) => {
						return {
							value: group.id,
							label: entities.decode( group.name ),
							adIds: group.ad_ids,
						};
					} );
					callback( groups );
				} )
				.catch( ( error ) => {
					console.log( error );
				} );

			}

			//
			// Handles a change of ad group
			//
			const onGroupChange = ( value ) => {

				// Render a random ad
				const shuffledAdIds = arrayShuffle( value.adIds );

				props.setAttributes( {
					group_id   : value.value,
					group_name : value.label,
					ad_id      : shuffledAdIds[0],
				} );

			}

			//
			// Handles changes on the timing
			//
			const onTimeChange = ( event ) => {
				let value = event.target.value.replace( /\D/g, '' );
				if ( value < 1 ) {
					value = '';
				}
				props.setAttributes( {
					time: value,
				} );
			}

			//
			// Handles changes of alignment buttons
			//
			const onAlignChange = ( value ) => {
				props.setAttributes( {
					align: value,
				} );
			}

			//
			// Handles changes on the Max Width Enabled checkbox
			//
			const onMaxWidthEnabledChange = () => {
				let maxWidth = '100';
				if ( props.attributes.max_width_enabled ) {
					maxWidth = '';
				}
				props.setAttributes( {
					max_width: maxWidth,
					max_width_enabled: ! props.attributes.max_width_enabled,
				} );
			}

			//
			// Handles changes on the Max Width field
			//
			const onMaxWidthChange = ( value ) => {
				props.setAttributes( {
					max_width: value,
				} );
			}

			//
			// Save a rendered ad
			//
			const onRenderedAd = ( value ) => {
				props.setAttributes( {
					rendered_ad: value,
				} );
			}

			//
			// Handle change on "random" checkbox
			//
			const onRandomChange = () => {
				props.setAttributes( {
					random: ! props.attributes.random,
				} );
			}

			//
			// Define default values of the select component
			//
			const defaultValue = {
				value: props.attributes.group_id,
				label: props.attributes.group_name,
			};

			return (
				<div
					className={ props.className }
					data-align={ props.attributes.align }
				>
					{ !! props.isSelected ? (
						<Placeholder
							label={ sprintf( __( '%s Rotating Ads', 'adsanity' ), 'AdSanity' ) }
						>
							<label>
								<span>{ __( 'Select an ad group:', 'adsanity' ) }</span>
								<AsyncSelect
									cacheOptions
									defaultOptions
									loadOptions={ loadOptions }
									defaultValue={ defaultValue }
									theme={ selectTheme }
									onChange={ onGroupChange }
								/>
							</label>

							<AddOnsWarning/>

							<hr className="adsanity-section-separator"/>

							<label>
								<span>{ __( 'Time (in seconds):', 'adsanity' ) }</span>
								<input
									type="text"
									value={ props.attributes.time }
									onChange={ onTimeChange }
								/>
							</label>

							<hr className="adsanity-section-separator"/>

							<AlignButtonGroup
								onAlignChange={ onAlignChange }
								propValue={ props.attributes.align }
							/>

							<hr className="adsanity-section-separator"/>

							<MaxWidthControl
								onMaxWidthEnabledChange={ onMaxWidthEnabledChange }
								onMaxWidthChange={ onMaxWidthChange }
								maxWidthEnabled={ props.attributes.max_width_enabled }
								maxWidth={ props.attributes.max_width }
							/>

							<hr className="adsanity-section-separator"/>

							<div className="adsanity-max-width-control">
								<div
									style={ {
										display: 'flex',
										justifyContent: 'center',
										marginTop: '10px',
									} }
								>
									<CheckboxControl
										label={ __( 'Order Randomly?', 'adsanity' ) }
										onChange={ onRandomChange }
										checked={ props.attributes.random }
										style={ {
											backgroundColor: props.attributes.random ? colors.primary : colors.white,
											borderColor: props.attributes.random ? colors.primary : '',
										} }
									/>
									<Tooltip
										text={ __( '(defaults to ordering by publish date)', 'adsanity-raw' ) }
									>
										<IconButton
											isDefault
											className="adsanity-tooltip-button"
											icon={ questionIcon }
											label={ __( 'Hover for more information', 'adsanity-raw' ) }
										/>
									</Tooltip>
								</div>
							</div>

							<AutomaticInclusion/>

						</Placeholder>
					) : (
						<div>
							<SingleAd
								adId={ props.attributes.ad_id }
								renderedAd={ props.attributes.rendered_ad }
								onRenderedAd={ onRenderedAd }
								maxWidthEnabled={ props.attributes.max_width_enabled }
								maxWidth={ props.attributes.max_width }
								align={ props.attributes.align }
							/>
						</div>
					) }
				</div>
			);

		},
		save() {
			// Rendered in PHP
			return null;
		},
	}
);
