<?php

/**
 * The settings-specific functionality of the plugin.
 *
 * @link       https://pixeljar.com
 * @since      1.0.0
 *
 * @package    AdSanity_VII
 * @subpackage AdSanity_VII/settings
 */

/**
 * Require the authentication class
 */
require_once plugin_dir_path( dirname( __FILE__ ) ) . 'settings/class-adsanity-vii-auth.php';

/**
 * The settings-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the settings-specific stylesheet and JavaScript.
 *
 * @package    AdSanity_VII
 * @subpackage AdSanity_VII/settings
 * @author     Pixel Jar <info@pixeljar.com>
 */

class AdSanity_VII_Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * The instance of the authentication class
	 */
	private $authentication;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->get_authentication_instance();

	}

	/**
	 * Create a new instance of the authentication class
	 */
	public function get_authentication_instance() {

		$this->authentication = new AdSanity_VII_Auth( $this->plugin_name, $this->version );

	}

	/**
	 * Enqueue scripts and styles for the submenu page
	 */
	public function scripts( $hook_suffix ) {

		global $post;

		if ( 'post-new.php' !== $hook_suffix &&
			 ! empty( $post ) &&
			 property_exists( $post, 'post_type' ) &&
			 'ads' !== $post->post_type ) {
			return;
		}

		wp_enqueue_script(
			$this->plugin_name . '-admin',
			plugin_dir_url( __FILE__ ) . 'dist/bundle.js',
			array( 'jquery', 'wp-color-picker' ),
			$this->version,
			true
		);

		wp_enqueue_style(
			$this->plugin_name . '-admin',
			plugin_dir_url( __FILE__ ) . 'dist/bundle.css',
			array( 'wp-color-picker' ),
			$this->version,
			'all'
		);

	}

	/**
	 * Adds an ad tab on the edit screen for vi
	 */
	public function ad_tab( $tabs ) {

		$tabs['vi'] = array(
			'label'       => 'VI',
			'active_tabs' => array(
				'adsanity-vii',
				'ad-size',
				'adsanity-notes',
			),
		);

		return $tabs;

	}

	/**
	 * Add a metabox with the form for vi
	 */
	public function meta_box() {

		add_meta_box(
			$this->plugin_name,
			'video intelligence',
			array( $this, 'metabox_cb' ),
			'ads',
			'normal',
			'high'
		);

	}

	/**
	 * Call back for the metabox
	 */
	public function metabox_cb( $post ) {

		$post_id = $post->ID;

		/**
		 * Pass variables into the view
		 */
		$iab_categories = file_get_contents( plugin_dir_path( dirname( __FILE__ ) ) . 'iab-categories.json' );
		$iab_categories = json_decode( $iab_categories );
		$iab_category_url = $this->authentication->vi_settings->iabCategoriesURL;
		$languages = $this->authentication->vi_settings->languages;
		$is_authenticated = $this->authentication->is_authenticated();
		$plugin_name = $this->plugin_name;
		$post_meta = get_post_meta( $post_id, $this->plugin_name, true );
		if ( empty( $post_meta ) ) {
			$post_meta = array();
		}

		if ( $is_authenticated ) {
			require_once plugin_dir_path( __FILE__ ) . 'partials/meta-box.php';
		} else {
			require_once plugin_dir_path( __FILE__ ) . 'partials/meta-box-not-logged-in.php';
		}

	}

	/**
	 * Manage saving video intelligence
	 */
	public function save_postdata( $post_id, $post ) {

		if ( ! array_key_exists( $this->plugin_name, $_POST ) ) {
			return;
		}

		if ( 'ads' !== $post->post_type ) {
			return;
		}

		$sanitized = $this->sanitize_array( $_POST['adsanity-vii'] );

		if ( empty( $sanitized ) ||
			 ! array_key_exists( 'adUnitType', $sanitized ) ||
			 ! array_key_exists( 'iabCategory', $sanitized ) ||
			 ! array_key_exists( 'language', $sanitized ) ) {
			return;
		}

		update_post_meta( $post_id, '_size', '336x242' );
		update_post_meta( $post_id, $this->plugin_name, $sanitized );

		$get_script_tag = $this->authentication->get_script( $sanitized );

		if ( ! $get_script_tag ) {
			// Add error notice if we could not create a script tag
			return add_filter( 'redirect_post_location', array( $this, 'add_error_query_arg' ), 10, 1 );
		}

		update_post_meta( $post_id, '_code', $get_script_tag );

	}

	/**
	 * Sanitize an array
	 */
	private function sanitize_array( $dirty ) {

		$clean = array();

		foreach ( $dirty as $key => $value ) {
			if ( ! $value ) {
				continue;
			}
			$clean[ sanitize_text_field( $key ) ] = sanitize_text_field( $value );
		}

		return $clean;

	}

	/**
	 * Add error to query arguments
	 */
	public function add_error_query_arg( $location ) {

		return add_query_arg( 'adsanity-vii-error', 'true', $location );

	}

	/**
	 * Show error notice if code was not generated
	 */
	public function admin_notices() {

		if ( ! isset( $_GET['adsanity-vii-error'] ) ) {
			return;
		}

		$text = sprintf(
			__( 'There was an error processing your request. Unable to create a %s ad unit.' ),
			'video intelligence'
		);

		?>
			<div class="notice notice-error">
				<p><strong><?php echo esc_html( $text ); ?></strong></p>
			</div>
		<?php
	}

	/**
	 * Return true if this ad is a vi ad
	 */
	public function active_tabs( $tab, $post_id ) {

		$vii_meta = get_post_meta( $post_id, $this->plugin_name, true );

		if ( $vii_meta ) {
			return true;
		}

		return $tab;

	}

	/**
	 * Adds new size specific to VI Video Ads
	 *
	 * @param array $sizes
	 * @return array $sizes
	 */
	public function vi_ad_sizes( $sizes = array() ) {

		$sizes['336x242'] = '336x242 - vi Story';

		return $sizes;

	}

	/**
	 * Modify the "Post updated" message
	 *
	 * @param array $messages array of all post updated messages
	 * @return array array of all post updated messages
	 */
	public function post_updated_messages( $messages ) {

		global $post;

		if ( 'ads' !== $post->post_type ) {
			return $messages;
		}

		$post_id = $post->ID;

		$vii_meta = get_post_meta( $post_id, $this->plugin_name, true );

		if ( ! $vii_meta ) {
			return $messages;
		}

		$scheduled_date = date_i18n( __( 'M j, Y @ H:i' ), strtotime( $post->post_date ) );

		$messages['ads'] = array(
			0 => '', // Unused. Messages start at index 1.
			1 => sprintf( esc_html__( 'Click %s[shortcode]%s to copy ad tag to the clipboard. Insert the tag into a page/post to launch a %s stories unit.', 'adsanity-vii' ), '<code>', '</code>', 'vi' ),
			2 => __( 'Custom field updated.' ),
			3 => __( 'Custom field deleted.' ),
			4 => __( 'Ad updated.', 'adsanity-vii' ),
			5 => isset($_GET['revision']) ? sprintf( __( 'Ad restored to revision from %s.', 'adsanity-vii' ), wp_post_revision_title( (int) $_GET['revision'], false ) ) : false,
			6 => sprintf( esc_html__( 'Click %s[shortcode]%s to copy ad tag to the clipboard. Insert the tag into a page/post to launch a %s stories unit.', 'adsanity-vii' ), '<code>', '</code>', 'vi' ),
			7 => __( 'Ad saved.' ),
			8 => __( 'Ad submitted.' ),
			9 => sprintf( __( 'Ad scheduled for: %s.', 'adsanity-vii' ), '<strong>' . $scheduled_date . '</strong>' ),
			10 => __( 'Ad draft updated.', 'adsanity-vii' ),
		);

		return $messages;

	}

	public static function changelog() {
		require_once plugin_dir_path( __FILE__ ) . 'partials/changelog.php';
	}

}
