<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://pixeljar.com
 * @since      1.0.0
 *
 * @package    AdSanity_VII
 * @subpackage AdSanity_VII/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    AdSanity_VII
 * @subpackage AdSanity_VII/includes
 * @author     Pixel Jar <info@pixeljar.com>
 */
class AdSanity_VII {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      AdSanity_VII_Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct() {
		if ( defined( 'ADSANITY_VII_VERSION' ) ) {
			$this->version = ADSANITY_VII_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'adsanity-vii';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_auth_hooks();
		$this->define_public_hooks();
		$this->define_settings_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Include the following files that make up the plugin:
	 *
	 * - AdSanity_VII_Loader. Orchestrates the hooks of the plugin.
	 * - AdSanity_VII_i18n. Defines internationalization functionality.
	 * - AdSanity_VII_Settings. Defines all hooks for the settings area.
	 * - AdSanity_VII_Public. Defines all hooks for the public side of the site.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies() {

		/**
		 * The class responsible for orchestrating the actions and filters of the
		 * core plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-adsanity-vii-loader.php';

		/**
		 * The class responsible for defining internationalization functionality
		 * of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-adsanity-vii-i18n.php';

		/**
		 * The class responsible for defining all actions that occur in the admin area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-adsanity-vii-admin.php';

		/**
		 * Class responsible for handling authentication with video intelligence
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'settings/class-adsanity-vii-auth.php';

		/**
		 * The class responsible for defining all actions that occur on ths public-facing area of the plugin.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-adsanity-vii-public.php';

		/**
		 * The class responsible for defining all actions that occur in the settings area.
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'settings/class-adsanity-vii-settings.php';

		/**
		 * Update engine
		 */
		require_once plugin_dir_path( dirname( __FILE__ ) ) . 'update-engine/remote-api.php';

		$this->loader = new AdSanity_VII_Loader();

	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the AdSanity_VII_i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale() {

		$plugin_i18n = new AdSanity_VII_i18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}

	/**
	 * Register all of the hooks related to the admin functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks() {

		$plugin_admin = new AdSanity_VII_Admin( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'scripts', 10, 1 );
		$this->loader->add_filter( 'adsanity_ad_edit_tabs', $plugin_admin, 'ad_tab', 10, 1 );
		$this->loader->add_action( 'add_meta_boxes', $plugin_admin, 'meta_box' );
		$this->loader->add_action( 'save_post', $plugin_admin, 'save_postdata', 20, 2 );
		$this->loader->add_action( 'admin_notices', $plugin_admin, 'admin_notices' );
		$this->loader->add_filter( 'adsanity_add_on_active_tab', $plugin_admin, 'active_tabs', 10, 2 );
		$this->loader->add_filter( 'adsanity_ad_sizes', $plugin_admin, 'vi_ad_sizes' );
		$this->loader->add_filter( 'post_updated_messages', $plugin_admin, 'post_updated_messages', 10, 1 );
		$this->loader->add_action( 'adsanity_changelog_screen', $plugin_admin, 'changelog' );

	}

	/**
	 * Register all of the hooks related to the video intelligence
	 * authentication functionality of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_auth_hooks() {

		$plugin_auth = new AdSanity_VII_Auth( $this->get_plugin_name(), $this->get_version() );

		$ajax_prefix = 'wp_ajax_' . $this->plugin_name . '-';

		$this->loader->add_action( $ajax_prefix . 'authenticate-user', $plugin_auth, 'authenticate_user' );
		$this->loader->add_action( $ajax_prefix . 'log-out', $plugin_auth, 'log_out' );

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks() {

		$plugin_public = new AdSanity_VII_Public( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'scripts' );
		$this->loader->add_action( 'enqueue_block_editor_assets', $plugin_public, 'scripts' );

	}

	/**
	 * Register all of the hooks related to the settings functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_settings_hooks() {

		$plugin_settings = new AdSanity_VII_Settings( $this->get_plugin_name(), $this->get_version() );

		$this->loader->add_action( 'admin_menu', $plugin_settings, 'submenu_page' );
		$this->loader->add_action( 'admin_enqueue_scripts', $plugin_settings, 'scripts', 10, 1 );

	}

	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run() {
		$this->loader->run();
	}

	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name() {
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    AdSanity_VII_Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader() {
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version() {
		return $this->version;
	}

}
