<?php

/**
 * Require the ads.txt class
 */
require_once plugin_dir_path( __FILE__ ) . 'class-adsanity-vii-ads-txt.php';

/**
 * Class that handles the authentication for video intelligence
 */
class AdSanity_VII_Auth {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * VI settings. These are endpoints for video intelligence
	 */
	public $vi_settings;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->get_vi_settings();

	}

	/**
	 * Get the settings for vi.
	 * This includes all of the endpoints
	 */
	private function get_vi_settings() {

		$transient_name = $this->plugin_name . '-endpoints';

		// Check to see if we have a transient in place for vi settings
		if ( ! $this->vi_settings = get_transient( $transient_name ) ) {
			// $vi_endpoint = 'https://dashboard-api-test.vidint.net/v1/api/widget/settings';
			$vi_endpoint = 'https://dashboard-api.vidint.net/v1/api/widget/settings';
			$request = wp_safe_remote_get( $vi_endpoint );
			if ( ! is_wp_error( $request ) ) {

				$this->vi_settings = json_decode( $request['body'] );

				set_transient( $transient_name, $this->vi_settings->data, DAY_IN_SECONDS );

			}
		}

	}

	/**
	 * Authenticate a user with video intelligence
	 */
	public function authenticate_user() {

		if ( ! array_key_exists( 'email', $_POST ) ||
		     ! array_key_exists( 'password', $_POST ) ) {
			return wp_send_json_error();
		}

		$email = sanitize_text_field( $_POST['email'] );
		$password = sanitize_text_field( $_POST['password'] );
		$request_body = json_encode( array(
			'email'    => $email,
			'password' => $password,
		) );

		$authentication = wp_safe_remote_post( $this->vi_settings->loginAPI, array(
			'headers' => array(
				'Content-Type' => 'application/json; charset=utf-8',
			),
			'body' => $request_body,
		) );

		if ( ! array_key_exists( 'response', $authentication ) ||
		     ! array_key_exists( 'code', $authentication['response'] ) ) {
			return wp_send_json_error();
		}

		if ( 200 !== $authentication['response']['code'] ) {
			return wp_send_json_error();
		}

		$body = json_decode( $authentication['body'] );
		$jwt_token = $body->data;

		// Update the option for jwt token
		update_option( $this->plugin_name . '_jwt_token', $jwt_token );

		// Handle ads.txt
		$this->handle_ads_txt();

		return wp_send_json_success();

	}

	/**
	 * Log a user out
	 */
	public function log_out() {

		// Remove the jwt option
		delete_option( $this->plugin_name . '_jwt_token' );

		return wp_send_json_success();

	}

	/**
	 * Update ads.txt
	 */
	public function handle_ads_txt() {

		$jwt_token = $this->retrieve_jwt();

		if ( ! $jwt_token ) {
			return false;
		}

		$request = wp_safe_remote_get( $this->vi_settings->adsTxtAPI, array(
			'headers' => array(
				'Authorization' => $jwt_token,
				'Content-Type'  => 'application/json; charset=utf-8',
			),
		) );

		if ( ! array_key_exists( 'response', $request ) ||
		     ! array_key_exists( 'code', $request['response'] ) ) {
			return false;
		}

		if ( 200 !== $request['response']['code'] ) {
			return false;
		}

		$body = json_decode( $request['body'] );
		$ads_txt_content = $body->data;

		new AdSanity_VII_Ads_Txt( $ads_txt_content );

	}

	/**
	 * Check is a user is authenticated
	 */
	public function is_authenticated() {

		// First, get the decoded JWT
		$decoded_jwt = $this->decode_jwt();

		if ( ! $decoded_jwt ) {
			return false;
		}

		// Make sure it has not expired
		if ( $decoded_jwt->exp < time() ) {
			// If expired, delete the jwt token option
			delete_option( $this->plugin_name . '_jwt_token' );
			return false;
		}

		return true;

	}

	/**
	 * Retrieve the JWT token
	 */
	public function retrieve_jwt() {

		$jwt_token = get_option( $this->plugin_name . '_jwt_token' );

		if ( ! $jwt_token ) {
			return false;
		}

		return $jwt_token;

	}

	/**
	 * Decode jwt
	 */
	private function decode_jwt() {

		$jwt_token = $this->retrieve_jwt();

		if ( ! $jwt_token ) {
			return false;
		}

		$jwt_token = explode( '.', $jwt_token );
		$base64 = base64_decode( $jwt_token[1] );

		return json_decode( $base64 );

	}

	/**
	 * Get the revenue data
	 */
	public function get_revenue_data() {

		$jwt_token = $this->retrieve_jwt();

		if ( ! $jwt_token ) {
			return false;
		}

		$request = wp_safe_remote_get( $this->vi_settings->revenueAPI, array(
			'headers' => array(
				'Authorization' => $jwt_token,
				'Content-Type'  => 'application/json; charset=utf-8',
			),
		) );

		if ( ! array_key_exists( 'response', $request ) ||
		     ! array_key_exists( 'code', $request['response'] ) ) {
			return false;
		}

		if ( 200 !== $request['response']['code'] ) {
			return false;
		}

		$body = json_decode( $request['body'] );
		return $body->data;

	}

	/**
	 * Get a script tag
	 */
	public function get_script( $data ) {

		$jwt_token = $this->retrieve_jwt();

		if ( ! $jwt_token ) {
			return false;
		}

		// Add the domain to data
		// $domain = str_replace( array( 'http://', 'https://' ), '', get_bloginfo( 'url' ) );
		$parsed_url = parse_url( home_url() );
		$data['domain'] = $parsed_url['host'];

		$request = wp_safe_remote_post( $this->vi_settings->jsTagAPI, array(
			'headers' => array(
				'Content-Type'  => 'application/json; charset=utf-8',
				'Authorization' => $jwt_token,
			),
			'body' => json_encode( $data ),
		) );

		if ( ! array_key_exists( 'response', $request ) ||
		     ! array_key_exists( 'code', $request['response'] ) ) {
			return false;
		}

		if ( 201 !== $request['response']['code'] ) {
			return false;
		}

		$body = json_decode( $request['body'] );

		$script = '<div class="adsanity-vi-unit"><script>' . $body->data . '</script></div>';

		return $script;

	}

}
