<?php

/**
 * Class that handles modifications
 * to ad groups to weight results
 */

class AdSanity_AW_Ad_Weight {

	function __construct() {}

	/**
	 * The actions and filters for this class
	 */
	public static function hooks() {

		add_filter( 'posts_join', array( __CLASS__, 'modify_join' ), 10, 2 );
		add_filter( 'posts_orderby', array( __CLASS__, 'modify_orderby' ), 10, 2 );
		add_filter( 'adsanity_raw_ids_in_group', array( __CLASS__, 'modify_rotating_ad_ids' ), 10, 1 );

	}

	/**
	 * Whether or not this query is for an ad group
	 */
	public static function is_ad_group( $query ) {

		if ( ! $query->is_tax( 'ad-group' ) || is_admin() ) {
			return false;
		}

		return true;

	}

	/**
	 * Modifies the query's JOIN statement
	 * to add in the ad weight metadata
	 */
	public static function modify_join( $join, $query ) {

		if ( ! self::is_ad_group( $query ) ) {
			return $join;
		}

		global $wpdb;

		return $join . $wpdb->prepare(
			"
			LEFT JOIN {$wpdb->postmeta} AS adsanity_wa_weight
			ON ( {$wpdb->posts}.ID = adsanity_wa_weight.post_id
			AND adsanity_wa_weight.meta_key = %s )
			",
			ADSANITY_WA_META_KEY
		);

	}

	/**
	 * Custom order by for ad groups
	 *
	 * Reference: http://www.kahunaburger.com/2008/10/13/selecting-random-weighted-records-from-mysql/
	 */
	public static function modify_orderby( $orderby, $query ) {

		if ( ! self::is_ad_group( $query ) ) {
			return $orderby;
		}

		global $wpdb;

		return "-LOG(1.0 - RAND()) / COALESCE( adsanity_wa_weight.meta_value, 50 )";

	}

	/**
	 * Modify the ad ids for a rotating ad widget
	 *
	 * @param  array $ids_in_group
	 * @return void
	 */
	public static function modify_rotating_ad_ids( $ids_in_group ) {

		// Boolean for whether or not this group should be reorganized
		$reorganize = false;

		/**
		 * Create an array with ad weights and ids, multiplied by weight
		 */
		$multiplied_ads = [];

		foreach ( $ids_in_group as $id ) {
			// Get the ad's weight or default of 50
			$weight = get_post_meta( $id, ADSANITY_WA_META_KEY, true );
			if ( ! $weight ) {
				$weight = 50;
			}

			// Only need to reorganize if at least one ad is not normal
			if ( 50 !== $weight ) {
				$reorganize = true;
			}

			// Get the multiplier based on ad weight
			$multiplier = 1;
			switch ( $weight ) {
				case 1:
					$multiplier = 1;
					break;
				case 25:
					$multiplier = 2;
					break;
				case 50:
					$multiplier = 3;
					break;
				case 75:
					$multiplier = 4;
					break;
				case 100:
					$multiplier = 5;
					break;
				default :
					break;
			}

			// Push ads to new array based on multiplier
			for ( $i = 0; $i < $multiplier; ++$i ) {
				$multiplied_ads[] = [
					'id'     => $id,
					'weight' => $weight,
				];
			}
		}

		// Create an array of new ad ids, randomized
		$new_ad_ids = [];

		do {
			// Get a random array key for multiplied ads array
			$random_array_key = rand( 0, count( $multiplied_ads ) - 1 );

			// If this is the beginning of the array
			if ( ! count( $new_ad_ids ) ) {
				$new_ad_ids[] = $multiplied_ads[ $random_array_key ]['id'];
				array_splice( $multiplied_ads, $random_array_key, 1 );
				continue;
			}

			// Conditional preventing back to back duplicates
			if ( $new_ad_ids[ count( $new_ad_ids ) - 1 ] !== $multiplied_ads[ $random_array_key ]['id'] ) {
				$new_ad_ids[] = $multiplied_ads[ $random_array_key ]['id'];
				array_splice( $multiplied_ads, $random_array_key, 1 );
				continue;
			}
		} while ( self::array_is_unique( $multiplied_ads, 'id' ) );

		// If the first and last values of match, remove it
		if ( $new_ad_ids[0] === $new_ad_ids[ count( $new_ad_ids ) - 1 ] ) {
			array_splice( $new_ad_ids, count( $new_ad_ids ) - 1, 1 );
		}

		if ( $reorganize ) {
			return $new_ad_ids;
		}

		return $ids_in_group;

	}

	/**
	 * Helper function for unique array
	 *
	 * @param  array $array  Input array
	 * @param  string $key  The key to test for
	 * @return bool  True if array contains two or more unique values, False if there are no values or only duplicate values
	 */
	public static function array_is_unique( $array, $key ) {

		if ( ! is_array( $array ) || ! count( $array ) ) {
			return false;
		}

		// Flatten the array to only include a one-dimensional array of the key's values
		$flattened = [];
		foreach ( $array as $value ) {
			$flattened[] = $value[ $key ];
		}

		// Make the array only unique values
		$unique = array_unique( $flattened );

		// If only one value remains, this does not contain unique value
		if ( 1 === count( $unique ) ) {
			return false;
		}

		return true;

	}

}

AdSanity_AW_Ad_Weight::hooks();
