<?php
if ( ! defined( 'ABSPATH' ) ) { die( 'You are not allowed to call this file directly.' ); }

/**
 * Ad group template tag
 * @param  array  $args accepted args:
 *                      - (bool) is_widget: outputs widget wrap/title
 *                      - (array) widget_args: only used in widget mode
 *                      - (string) title: only used in widget mode
 *                      - (array) group_ids: array of ad-group term ids
 *                      - (int) num_ads: number of ads to show
 *                      - (int) num_columns: number of columns
 *                      - (bool) return: whether to return or directly output
 * @return string       if return is set to true, will return the html. used in shortcodes.
 */
function adsanity_show_ad_group( $args = array() ) {

	$defaults = array(
		'is_widget'		=> false,
		'widget_args'	=> array(),
		'title'			=> false,
		'group_ids'		=> array(),
		'num_ads'		=> 0,
		'num_columns'	=> 0,
		'return'		=> false,
		'max_width'		=> false,
		'align'         => 'alignnone',
	);
	$values = wp_parse_args( $args, $defaults );
	extract( $values, EXTR_SKIP );

	$group_ids = apply_filters( 'adsanity_hide_ad_group', $group_ids );
	$values['group_ids'] = $group_ids;

	$post__not_in = apply_filters( 'adsanity_hide_ad_in_group', array(), $group_ids );
	$values['post__not_in'] = $post__not_in;

	$num_ads = intval( $num_ads );
	$num_columns = intval( $num_columns );

	if ( count( $group_ids ) < 1 ) {
		return false;
	}

	if ( $return ) {
		ob_start();
	}

	$now = time();

	// Get the cached version of the ad first.
	$cache_key = sprintf(
		'group-of-ads-%s',
		AdSanityUtility::hash( serialize( $values ) )
	);
	$ads = get_transient( $cache_key );
	if ( false === $ads ) {
		delete_transient( $cache_key );

		$ads_args = array(
			'post_type' => 'ads',
			'tax_query' => array(
				array(
					'taxonomy' => 'ad-group',
					'field' => 'id',
					'terms' => $group_ids,
				)
			),
			'meta_query' => array(
				array(
					'key' => '_start_date',
					'value' => $now,
					'type' => 'numeric',
					'compare' => '<=',
				),
				array(
					'key' => '_end_date',
					'value' => $now,
					'type' => 'numeric',
					'compare' => '>=',
				)
			),
			'posts_per_page' => $num_ads,
			'orderby' => 'rand',
			'no_found_rows' => true,
		);

		if ( ! empty( $post__not_in ) ) {
			$ads_args['post__not_in'] = $post__not_in;
		}

		$ads = new WP_Query( $ads_args );

		// Cache for 60 seconds.
		set_transient( $cache_key, $ads, MINUTE_IN_SECONDS );

	}

	if ( $ads->have_posts() ) {

		if ( $is_widget ) {
			echo $widget_args['before_widget'];

			if ( $title && ! empty( $title ) ) {
				echo $widget_args['before_title'];
				echo esc_html( $title );
				echo $widget_args['after_title'];
			}
		}

		printf( '<div class="ad-%s">', esc_attr( $align ) );

		$index = 0;
		while ( $ads->have_posts() ) {

			$ads->the_post();

			// Count the view.
			adsanity_view( get_the_ID() );

			$size = get_post_meta( get_the_ID(), '_size', true );

			// Override in a parent or child theme.
			$custom_template = locate_template(
				array(
					sprintf( 'ad-%s.php', get_the_ID() ),
					sprintf( 'ad-%s.php', $size ),
					'ad.php',
				)
			);

			if ( 0 === $index || 0 === $index % $num_columns ) {
				echo '<div class="ad-row">';
			}

			if ( ! empty( $custom_template ) ) {
				require( $custom_template );
			} else {
				require( ADSANITY_THEME . 'ad.php' );
			}

			if ( 0 === ( $index + 1 ) % $num_columns ) {
				echo '</div>';
			}

			$index++;

		}

		// Close up the ad-row div if necessary.
		if ( 0 !== $index % $num_columns ) {
			echo '</div>';
		}

		// Close up the alignment div.
		echo '</div>';

		if ( $is_widget ) {
			echo $widget_args['after_widget'];
		}

		wp_reset_postdata();
	}

	if ( $return ) {
		$output = ob_get_contents();
		ob_end_clean();
		return $output;
	}

}

/**
 * Single ad template tag
 * @param  array  $args accepted args:
 *                      - (bool) is_widget: outputs widget wrap/title
 *                      - (array) widget_args: only used in widget mode
 *                      - (string) title: only used in widget mode
 *                      - (int) post_id: ID of ad to show
 *                      - (string) align: alignnone|alignleft|aligncenter|alignright
 *                      - (bool) return: whether to return or directly output
 * @return string       if return is set to true, will return the html. used in shortcodes.
 */
function adsanity_show_ad( $args = array() ) {

	$defaults = array(
		'is_widget'   => false,
		'widget_args' => array(),
		'title'       => false,
		'post_id'     => 1,
		'align'       => false,
		'return'      => false,
		'max_width'   => false,
		'track_this'  => true,
	);
	$values = wp_parse_args( $args, $defaults );
	extract( $values, EXTR_SKIP );

	if ( intval( $post_id ) != $post_id ) {
		return false;
	}

	$hide_ad = apply_filters( 'adsanity_hide_ad', false, $post_id );
	if ( $hide_ad ) {
		return;
	}

	// Sometimes max width is enabled, but no value is passed
	// for the actual max width. This should default to 100
	if (
		isset( $args['max_width_enabled'] ) &&
		$args['max_width_enabled'] &&
		! $max_width
	) {
		$max_width = 100;
	}

	if ( $return ) {
		ob_start();
	}

	$now = time();

	// Get the cached version of the ad first.
	$cache_key = sprintf( 'single-ad-%s', AdSanityUtility::hash( serialize( $values['post_id'] ) ) );
	$ad = get_transient( $cache_key );
	if ( false === $ad ) {
		delete_transient( $cache_key );

		// Get the ad using start/end dates.
		$ad = new WP_Query(
			array(
				'p' => $post_id,
				'post_type' => 'ads',
				'meta_query' => array(
					array(
						'key'		=> '_start_date',
						'value'		=> $now,
						'type'		=> 'numeric',
						'compare'	=> '<=',
					),
					array(
						'key'		=> '_end_date',
						'value'		=> $now,
						'type'		=> 'numeric',
						'compare'	=> '>=',
					),
				),
				'posts_per_page' => 1,
				'no_found_rows' => true,
			)
		);

		$transient_expiration = DAY_IN_SECONDS / 2;

		//
		// If the end date for this ad is in less than
		// 12 hours, use that for the transient expiration
		//
		// Don't use the standard loop
		//
		if ( $ad->have_posts() ) {
			$post_id = $ad->posts[0]->ID;
			$end_date = get_post_meta( $post_id, '_end_date', true );

			if ( $end_date ) {

				if ( ( $end_date - time() ) < $transient_expiration ) {
					$transient_expiration = $end_date - time();
				}

			}

		}

		set_transient( $cache_key, $ad, $transient_expiration ); // Cache for 12 hours or end date of this ad
	}
	if ( $ad->have_posts() ) {
		$ad->the_post();

		// Count the view.
		if ( $track_this ) {
			adsanity_view( get_the_ID() );
		}

		if ( $is_widget ) {
			echo $widget_args['before_widget'];

			if ( $title && !empty( $title ) ) {
				echo $widget_args['before_title'];
				echo esc_html( $title );
				echo $widget_args['after_title'];
			}
		}

		$size = get_post_meta( get_the_ID(), '_size', true );

		// override in a parent or child theme
		$custom_template = locate_template(
			array(
				sprintf( 'ad-%s.php', get_the_ID() ),
				sprintf( 'ad-%s.php', $size ),
				'ad.php',
			)
		);
		if ( ! empty( $custom_template ) ) {
			require( $custom_template );
		} else {
			require( ADSANITY_THEME . 'ad.php' );
		}

		if ( $is_widget ) {
			echo $widget_args['after_widget'];
		}

		wp_reset_postdata();

	}

	if ( $return ) {

		$output = ob_get_contents();
		ob_end_clean();
		return $output;

	}

}
